/*************************************************************************
 * File: $Source$
 * Author: Jan Newmarch
 * Last modified: $Date$
 * Version: $Revision$
 * Purpose: Implement the Pane object, that holds a collection of 
 *	PaneButtons, each representing a file
 *
 * Revision history:
 ************************************************************************/ 

/*************************************************************************
 * System includes
 ************************************************************************/ 
#include <Xm/Xm.h>

/*************************************************************************
 * Local includes
 ************************************************************************/ 
#include "const.h"
#include "types.h"
#include "xmvararg.h"
#include "Pane.h"
#include "PaneB.h"
#include "debug_malloc.h"

#ifdef RDD
#include "rdd.h"
#endif /* RDD */

/*************************************************************************
 * Functions exported
 ************************************************************************/ 

/*************************************************************************
 * Variables exported
 ************************************************************************/ 

/*************************************************************************
 * Extern variables
 ************************************************************************/ 
#ifdef RDD
extern XtTranslations trans_table;
#endif /* RDD */

#ifdef MOTIF_DD
extern XtTranslations trans_table;
#endif /* MOTIF_DD */

extern XtTranslations configure_trans_table;


/*************************************************************************
 * Extern functions
 ************************************************************************/ 
#ifdef MOTIF_DD
extern  void MotifDDRegisterToolbar (
#ifdef UseFunctionPrototypes
	Widget button
#endif
);
#endif /* MOTIF_DD */

#ifdef RDD
extern void DirDrop (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer call, XtPointer client_data
#endif
);
extern void ExecutableDrop (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer call, XtPointer client_data
#endif
);
extern void MainWindowDrop (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer call, XtPointer client_data
#endif
);
extern void ToolbarDrop (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer call, XtPointer client_data
#endif
);
#endif /* RDD */


/*************************************************************************
 * Forward functions
 ************************************************************************/ 

/*************************************************************************
 * Local variables
 ************************************************************************/ 
#define BUTTON_INCR 64

/*************************************************************************
 * Function:
 * Purpose         :
 * In parameters   :
 * Out parameters  :
 * Side effects    :
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 

#ifdef ALLOW_ROWCOL_RESIZE
/*************************************************************************
 * Function: PaneResized ()
 * Purpose: the scrolled window containing files has changed in size.
 *      Need to rearrange widgets in it.
 * In parameters: w, event, args, num_args
 * Function result: 
 * Precondition: 
 * Postcondition: number of columns is maximum for new width
 ************************************************************************/

/* ARGSUSED */ 
void
PaneResized 
#ifdef UseFunctionPrototypes
        (Widget w, XEvent *event, char **args, Cardinal *num_args)
#else
        (w, event, args, num_args)
        Widget w;
        XEvent *event;
        char **args;
        Cardinal *num_args;

#endif
{       Dimension pane_width, button_width;
        int num_columns;
        Widget button = NULL;
        dir_pane_info *dpi = NULL;

#ifdef DEBUG
        {       XConfigureEvent *ev = (XConfigureEvent *) event;
                fprintf (stderr, "resize on scrolled widget to width %d height %d\n",
                        ev -> width, ev -> height);
        }
#endif

        XtVaGetValues (w,
                        XmNuserData, &dpi,
                        XmNwidth, &pane_width,
                        NULL);

        if (dpi -> files_buttons_last != -1)
                button = dpi -> files_buttons [dpi -> files_buttons_last];
        else if (dpi -> dirs_buttons_last != -1)
                button = dpi -> dirs_buttons [dpi -> dirs_buttons_last];
        else if (dpi -> executables_buttons_last != -1)
                button = dpi -> executables_buttons [dpi -> executables_buttons_last];
        else return;


        XtVaGetValues (button, XmNwidth, &button_width, NULL);

        num_columns = (int) (pane_width / button_width);

        /* sanity check on column size */
        if (num_columns < 1) num_columns = 1;

#ifdef DEBUG
        fprintf (stderr, "number of columns in panes set to %d\n",
                        num_columns);
#endif /* DEBUG */

        /* set the numColumns in each pane */
        XtVaSetValues (dpi -> executables_pane,
                        XmNnumColumns, (XtArgVal) num_columns,
                        XmNpacking, (XtArgVal) XmPACK_COLUMN,
                        NULL);
        XtVaSetValues (dpi -> files_pane,
                        XmNnumColumns, (XtArgVal) num_columns,
                        XmNpacking, (XtArgVal) XmPACK_COLUMN,
                       NULL);
        XtVaSetValues (dpi -> dirs_pane,
                        XmNnumColumns, (XtArgVal) num_columns,
                        XmNpacking, (XtArgVal) XmPACK_COLUMN,
                        NULL);
}
#endif /* ALLOW_ROWCOL_RESIZE */


/*************************************************************************
 * Function	   : PaneCreate
 * Purpose         : create a pane object to hold buttons for files
 * In parameters   : parent, name, filter_label
 * Out parameters  : pane, filter
 * Side effects    :
 * Function returns: created pane
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 
Widget
PaneCreate
#ifdef UseFunctionPrototypes
	(Widget parent, Widget *pane, char *name, Widget *filter, 
		char *filter_label, dir_pane_info *dpi)
#else
	(parent, pane, name, filter, filter_label, dpi)
	Widget parent;
	Widget *pane;
	char   *name;
	Widget *filter;
	char   *filter_label;
	dir_pane_info *dpi;
#endif
{
	Widget rc, sw;

        rc = XmVaCreatePanedWindow (parent, "pane_rc", 
                        XmNsashHeight, (XtArgVal) 1,
                        XmNseparatorOn, (XtArgVal) False,
                        XmNspacing, (XtArgVal) 0,
                        XmNuserData, (XtArgVal) dpi,
                        NULL);

        *filter = XmVaCreateLabel (rc,
                                        filter_label,
                                        NULL);

        sw = XmVaCreateScrolledWindow (rc, "pane_sw",
                        XmNscrollingPolicy, (XtArgVal) XmAUTOMATIC,
                        XmNuserData, (XtArgVal) dpi,
                        NULL);

#ifdef ALLOW_ROWCOL_RESIZE
        /* look for configure (resize) events on one of the panes */
        XtOverrideTranslations (sw, configure_trans_table);
#endif /* ALLOW_ROWCOL_RESIZE */

        *pane = XmVaCreateRowColumn (sw, name,
                                       NULL);
        XtVaSetValues (sw,
                        XmNworkWindow, (XtArgVal) *pane,
                        NULL);

	XtManageChild (*pane);
	XtManageChild (*filter);
	XtManageChild (rc);
	XtManageChild (sw);

	return rc;
}
 
/*************************************************************************
 * Function 	   : DirPaneUnshow
 * Purpose         : remove pane from display, preparatory to rebuilding list
 * In parameters   : dpi
 * Out parameters  :
 * Side effects    : unmaps window
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 
void
DirPaneUnshow
#if UseFunctionPrototypes
	(dir_pane_info *dpi)
#else
	(dpi)
	dir_pane_info *dpi;
#endif
{
        XtUnmapWidget (XtParent (dpi -> dirs_pane));
        XtUnmanageChildren (dpi -> dirs_buttons,
                            dpi -> dirs_buttons_last + 1);
}
 
/*************************************************************************
 * Function 	   : DirPaneShow
 * Purpose         : put pane bac on display
 * In parameters   : dpi
 * Out parameters  :
 * Side effects    : maps window
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 
void
DirPaneShow
#if UseFunctionPrototypes
	(dir_pane_info *dpi)
#else
	(dpi)
	dir_pane_info *dpi;
#endif
{
        XtManageChildren (dpi -> dirs_buttons,
                            dpi -> dirs_buttons_last + 1);
        XtMapWidget (XtParent (dpi -> dirs_pane));
}
 
/*************************************************************************
 * Function 	   : ExecutablePaneUnshow
 * Purpose         : remove pane from display, preparatory to rebuilding list
 * In parameters   : dpi
 * Out parameters  :
 * Side effects    : unmaps window
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 
void
ExecutablePaneUnshow
#if UseFunctionPrototypes
	(dir_pane_info *dpi)
#else
	(dpi)
	dir_pane_info *dpi;
#endif
{
        XtUnmapWidget (XtParent (dpi -> executables_pane));
        XtUnmanageChildren (dpi -> executables_buttons,
                            dpi -> executables_buttons_last + 1);
}
 
/*************************************************************************
 * Function 	   : ExecutablePaneShow
 * Purpose         : put pane back on display
 * In parameters   : dpi
 * Out parameters  :
 * Side effects    : maps window
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 
void
ExecutablePaneShow
#if UseFunctionPrototypes
	(dir_pane_info *dpi)
#else
	(dpi)
	dir_pane_info *dpi;
#endif
{
        XtManageChildren (dpi -> executables_buttons,
                            dpi -> executables_buttons_last + 1);
        XtMapWidget (XtParent (dpi -> executables_pane));
}
 
/*************************************************************************
 * Function 	   : FilePaneUnshow
 * Purpose         : remove pane from display, preparatory to rebuilding list
 * In parameters   : dpi
 * Out parameters  :
 * Side effects    : unmaps window
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 
void
FilePaneUnshow
#if UseFunctionPrototypes
	(dir_pane_info *dpi)
#else
	(dpi)
	dir_pane_info *dpi;
#endif
{
        XtUnmapWidget (XtParent (dpi -> files_pane));
        XtUnmanageChildren (dpi -> files_buttons,
                            dpi -> files_buttons_last + 1);
}
 
/*************************************************************************
 * Function 	   : FilePaneShow
 * Purpose         : put pane back on display
 * In parameters   : dpi
 * Out parameters  :
 * Side effects    : maps window
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 
void
FilePaneShow
#if UseFunctionPrototypes
	(dir_pane_info *dpi)
#else
	(dpi)
	dir_pane_info *dpi;
#endif
{
        XtManageChildren (dpi -> files_buttons,
                            dpi -> files_buttons_last + 1);
        XtMapWidget (XtParent (dpi -> files_pane));
}

/*************************************************************************
 * Function	   : DirPaneAddFile
 * Purpose         : add a file to the list of files in pane
 * In parameters   : filename, dpi
 * Out parameters  :
 * Side effects    :
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 
void
DirPaneAddFile
#if UseFunctionPrototypes
	(char * filename, dir_pane_info *dpi)
#else
	(filename, dpi)
	char *filename;
	dir_pane_info *dpi;
#endif
{
	int dirs_max;
	Widget *pbutton;

	dirs_max = dpi -> dirs_buttons_max;
	dpi -> dirs_buttons_last ++;
	if (dpi -> dirs_buttons_last >= dirs_max)
	{	int i;
		int new_size = dirs_max + BUTTON_INCR;
		/* grow the array cos we've run out of room */
		dpi -> dirs_buttons = (Widget *) XtRealloc (
				(char *) dpi -> dirs_buttons,
				new_size * sizeof (Widget));

		/* NULL out the extra elements */
		for (i = dirs_max; i < new_size; i++)
			dpi -> dirs_buttons [i] = NULL;
		dpi -> dirs_buttons_max  =
		dirs_max = new_size;
	}

	/* point to next widget in array */
	pbutton = dpi -> dirs_buttons + dpi -> dirs_buttons_last;
	if (*pbutton == NULL)
	{	/* fill in new button */
		* pbutton = PaneButtonCreate (filename,
                                                        "dir_pane_button", dpi,
                                                        dpi -> dirs_pane,
                                                        DIR_TYPE);

#ifdef RDD
		rddAddDropHandler (*pbutton, DirDrop, NULL);
#endif /* RDD */
	}
	else PaneButtonResetLabel (*pbutton,
                                                filename,
                                                DIR_TYPE);

}

/*************************************************************************
 * Function	   : ExecutablePaneAddFile
 * Purpose         : add a file to the list of files in pane
 * In parameters   : filename, dpi
 * Out parameters  :
 * Side effects    :
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 
void
ExecutablePaneAddFile
#if UseFunctionPrototypes
	(char * filename, dir_pane_info *dpi)
#else
	(filename, dpi)
	char *filename;
	dir_pane_info *dpi;
#endif
{
	int executables_max;
	Widget *pbutton;

	executables_max = dpi -> executables_buttons_max;
	dpi -> executables_buttons_last ++;
	if (dpi -> executables_buttons_last >= executables_max)
	{	int i;
		int new_size = executables_max + BUTTON_INCR;
		/* grow the array cos we've run out of room */
		dpi -> executables_buttons = (Widget *) XtRealloc (
				(char *) dpi -> executables_buttons,
				new_size * sizeof (Widget));

		/* NULL out the extra elements */
		for (i = executables_max; i < new_size; i++)
			dpi -> executables_buttons [i] = NULL;
		dpi -> executables_buttons_max  =
		executables_max = new_size;
	}

	/* point to next widget in array */
	pbutton = dpi -> executables_buttons + dpi -> executables_buttons_last;
	if (*pbutton == NULL)
	{	/* fill in new button */
		* pbutton = PaneButtonCreate (filename,
                                                        "executable_pane_button", dpi,
                                                        dpi -> executables_pane,
                                                        EXECUTABLE_TYPE);

#ifdef RDD
		rddAddDropHandler (*pbutton, ExecutableDrop, NULL);
#endif /* RDD */
	}
	else PaneButtonResetLabel (*pbutton,
                                                filename,
                                                EXECUTABLE_TYPE);
}

/*************************************************************************
 * Function	   : FilePaneAddFile
 * Purpose         : add a file to the list of files in pane
 * In parameters   : filename, dpi
 * Out parameters  :
 * Side effects    :
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 
void
FilePaneAddFile
#if UseFunctionPrototypes
	(char * filename, dir_pane_info *dpi)
#else
	(filename, dpi)
	char *filename;
	dir_pane_info *dpi;
#endif
{
	int files_max;
	Widget *pbutton;

	files_max = dpi -> files_buttons_max;
	dpi -> files_buttons_last ++;
	if (dpi -> files_buttons_last >= files_max)
	{	int i;
		int new_size = files_max + BUTTON_INCR;
		/* grow the array cos we've run out of room */
		dpi -> files_buttons = (Widget *) XtRealloc (
				(char *) dpi -> files_buttons,
				new_size * sizeof (Widget));

		/* NULL out the extra elements */
		for (i = files_max; i < new_size; i++)
			dpi -> files_buttons [i] = NULL;
		dpi -> files_buttons_max  =
		files_max = new_size;
	}

	/* point to next widget in array */
	pbutton = dpi -> files_buttons + dpi -> files_buttons_last;
	if (*pbutton == NULL)
	{	/* fill in new button */
		* pbutton = PaneButtonCreate (filename,
                                                        "file_pane_button", dpi,
                                                        dpi -> files_pane,
                                                        FILE_TYPE);

#ifdef RDD
		rddAddDropHandler (*pbutton, DirDrop, NULL);
#endif /* RDD */
	}
	else PaneButtonResetLabel (*pbutton,
                                         filename,
                                         FILE_TYPE);
}
