/*************************************************************************
 * File: $Source$
 * Author: Jan Newmarch
 * Last modified: $Date$
 * Version: $Revision$
 * Purpose: implement the PaneButton object, which is a graphical
 *	display + info about a file in the current directory
 *
 * Revision history:
 *	 2 Apr 93	file created by cutting code from applicat.c
 *			and buttoncb.c
 ************************************************************************/ 

#include "copyright.h"

/*************************************************************************
 * System includes
 ************************************************************************/ 
#include <stdio.h>

/*************************************************************************
 * Local includes
 ************************************************************************/ 
#include "const.h"
#include "types.h"
#include "xmvararg.h"
#include "RegExp.h"
#include "ToolbarB.h"

/*************************************************************************
 * Functions exported
 ************************************************************************/ 

/*************************************************************************
 * Variables exported
 ************************************************************************/ 

/*************************************************************************
 * Extern variables
 ************************************************************************/ 
extern GC gc;
extern GC gc_reversed;
extern file_action *actions;
extern XmString empty_string;

#ifdef RDD
extern XtTranslations trans_table;
#endif /* RDD */

#ifdef MOTIF_DD
extern XtTranslations trans_table;
#endif /* MOTIF_DD */


/*************************************************************************
 * Extern functions
 ************************************************************************/ 

/*************************************************************************
 * Forward functions
 ************************************************************************/ 

/*************************************************************************
 * Local variables
 ************************************************************************/ 

/*************************************************************************
 * Function:
 * Purpose         :
 * In parameters   :
 * Out parameters  :
 * Side effects    :
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 


/*************************************************************************
 * Function: PaneButtonInvertColours ()
 * Purpose: swap  the fg and bg of the label and pixmap
 * In parameters: w
 * Out parameters:
 * Side effects: pne_button_info of button has label, pixmap swapped
 *      with inverted versions
 * Precondition:
 * Postcondition: label shows inverted.
 ************************************************************************/ 
void
PaneButtonInvertColours 
#ifdef UseFunctionPrototypes
        (Widget w)
#else
        (w)
        Widget w;

#endif
{       
        pane_button_info *pbi = NULL;
        Pixmap tmp_pixmap;
        GC tmp_gc = NULL;

        if (w == NULL)
                return;
        XtVaGetValues (w,
                XmNuserData, (XtArgVal) &pbi,
                NULL);
        
        /* swap the gc and pixmap with reversed forms */
        tmp_pixmap = pbi -> pixmap;
        pbi -> pixmap = pbi -> pixmap_reversed;
        pbi -> pixmap_reversed = tmp_pixmap;

        tmp_gc = pbi -> gc;
        pbi -> gc = pbi -> gc_reversed;
        pbi -> gc_reversed = tmp_gc;

        /* set the reversed pixmap in the button */
        XtVaSetValues (w,
                XmNlabelPixmap, (XtArgVal) pbi -> pixmap,
                NULL);

        /* and force a redisplay to redraw name */
        XClearArea (XtDisplay (w), XtWindow (w),
                        0, 0, 0, 0, True);
}


/*************************************************************************
 * Function: PaneButtonPressed ()
 * Purpose: handle a button press in any of the panes i.e. a file has
 *          been selected
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition: a button for a file, executable, or 
 *               directory has been pressed
 * Postcondition: toolbar buttons set up to reflect actions for this 
 *               selected file
 * Warning: this function is called by others that set last 2 params
 *      to NULL
 ************************************************************************/ 

/* ARGSUSED */
void
PaneButtonPressed 
#ifdef UseFunctionPrototypes
        (Widget w, XtPointer client_data, XtPointer call_data)
#else
        (w, client_data, call_data)
        Widget w;
        XtPointer client_data;
        XtPointer call_data;

#endif
{       
        pane_button_info *pbi = NULL;
        tool_button_info *tbi = NULL;
        file_action *pfa = NULL;
        action_pair *pap = NULL;
        XmString xmstr = NULL;
        int button;
        Widget tool_button = NULL;

#ifdef DEBUG
        fprintf (stderr, "button pressed\n");
#endif
        /* get the user data */
        XtVaGetValues (w,
                        XmNuserData, (XtArgVal) &pbi,
                        NULL);

        /* set the last selected back to normal and invert this one */
        if (pbi -> dpi -> file_selected != NULL)
                PaneButtonInvertColours (pbi -> dpi -> file_button_selected);
        PaneButtonInvertColours (w);
        
        /* set the file selected */
        pbi -> dpi -> file_selected = pbi -> name;
        pbi -> dpi -> file_button_selected = w;
        pbi -> dpi -> executable_selected = NULL;
        pbi -> dpi -> dir_selected = NULL;

        /* find file pattern matching this name */
        pfa = actions;
        while (pfa != NULL)
                if (pfa -> file_type == pbi -> file_type &&
                        RegExpMatch (pbi -> name, pfa -> fsm_ptr))
                        break;
                else    pfa = pfa -> next;

        /* description of this file type */
        if (pfa != NULL)
                pbi -> dpi -> description =  pfa -> description;
        else    pbi -> dpi -> description =  NULL;
        
	ToolbarSetActions (pbi -> dpi -> files_toolbar, pfa);


#ifdef RDD
        /* set the drag cursor */
        {       Window root;
                int x, y;
                unsigned int width, height;
                unsigned int border_width, depth;

                XGetGeometry (XtDisplay (pbi -> dpi -> toplevel),
                                pbi -> pixmap,
                                &root,
                                &x, &y, &width, &height,
                                &border_width, &depth);
                rddSetDragPixmap (pbi -> pixmap, width, height);
        }
#endif
}


/*************************************************************************
 * Function: PaneButtonReleased ()
 * Purpose: handle a button release in any of the panes i.e. a file has
 *          been selected. Check if it is a multiclick, and if so
 *          execute default action.
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition: a button for a file, executable, or 
                 directory has been released
 ************************************************************************/ 

/* ARGSUSED */
void
PaneButtonReleased 
#ifdef UseFunctionPrototypes
        (Widget w, XtPointer client_data, XtPointer call_data)
#else
        (w, client_data, call_data)
        Widget w;
        XtPointer client_data;
        XtPointer call_data;

#endif
{       pane_button_info *pbi = NULL;
        XmDrawnButtonCallbackStruct *c_data=
		(XmDrawnButtonCallbackStruct *) call_data;

#ifdef DEBUG
        fprintf (stderr, "button released\n");
#endif
        /* get the user data */
        XtVaGetValues (w,
                        XmNuserData, (XtArgVal) &pbi, NULL);
        /* if it was a double click, execute
           the default action. default action is in
           the first toolbar button, if sensitive */
        if ((c_data -> click_count) > 1 &&
                XtIsSensitive ((pbi -> dpi -> files_toolbar) [0]))
        {       /* set the busy cursor as a signal that something happens */
                BusyCursor (pbi -> dpi -> toplevel);

                /* fake default action on button */
                ToolbarButtonPressed ( (pbi -> dpi -> files_toolbar) [0],
                                        NULL, NULL);
                UnBusyCursor (pbi -> dpi -> toplevel);
        }
}


/*************************************************************************
 * Function: PaneButtonExposed ()
 * Purpose: handle expose events on file buttons
 * In parameters: button, client_data, call_data
 * Function returns:
 * Precondition: expose event generated for w
 * Postcondition: label redrawn
 *************************************************************************/

/* ARGSUSED */
static void
PaneButtonExposed 
#ifdef UseFunctionPrototypes
        (Widget button, XtPointer client_data, XtPointer call_data)
#else
        (button, client_data, call_data)
        Widget button;
        XtPointer client_data, call_data;

#endif
{       pane_button_info *pbi = NULL;
        Dimension height;

        XtVaGetValues (button,
                        XmNuserData, (XtArgVal) &pbi,
                        XmNheight, (XtArgVal) &height,
                        NULL);

        XDrawImageString (XtDisplay (button),
                        XtWindow (button),
                        pbi -> gc, 0, (int) height - 5, 
                        pbi -> name,
                        strlen (pbi -> name));
}

/*************************************************************************
 * Function: PaneButtonSetPixmap ()
 * Purpose: set the visual pixmap for this file type
 * In parameters: button, name, file_type
 * Function returns:
 * Side effects: pbi for this button has pixmap, insensitive_pixmap set
 * Precondition: none 
 * Postcondition: pixmap set as labelPixmap resource
 *************************************************************************/
static void
PaneButtonSetPixmap 
#ifdef UseFunctionPrototypes
        (Widget button, char *name, char file_type)
#else
        (button, name, file_type)
        Widget button;
        char *name;
        char file_type;

#endif
{
        Pixmap pixmap, pixmap_reversed;
        file_action *pfa = NULL;
        Pixel fg, bg;
        pane_button_info *pbi = NULL;
        char pixmap_path[MAXPATHLEN + 2];

        /* find existing colours used in button */
        XtVaGetValues (button,
                        XmNforeground, &fg,
                        XmNbackground, &bg,
                        NULL);

        /* find file pattern matching this name */
        pfa = actions;
        while (pfa != NULL)
                if (pfa -> file_type == file_type &&
                        RegExpMatch (name, pfa -> fsm_ptr))
                        break;
                else    pfa = pfa -> next;

        if (pfa == NULL)
        {
                fprintf (stderr, "cant find pattern for %s\n", name);
                return;
        }

        /* destroy old pixmaps */
        XtVaGetValues (button,
                        XmNuserData, (XtArgVal) &pbi,
                        NULL);
        XmDestroyPixmap (XtScreen (button), pbi -> pixmap);
        XmDestroyPixmap (XtScreen (button), pbi -> pixmap_reversed);

        /* find the pixmap for this file */
#ifdef PIXMAPDIR
        strcpy (pixmap_path, PIXMAPDIR);
        strcat (pixmap_path, "/");
        strcat (pixmap_path, pfa -> pixmap);
#else
        strcpy (pixmap_path, pfa -> pixmap);
#endif
        pixmap = XmGetPixmap ( XtScreen (button),
                                pixmap_path, fg, bg);
        pixmap_reversed = XmGetPixmap ( XtScreen (button),
                                pixmap_path, bg, fg);

        if (pixmap == XmUNSPECIFIED_PIXMAP)
        {       fprintf (stderr, "cant find pixmap %s\n", pfa -> pixmap);
                /* reasonable default */
                pixmap = XmGetPixmap ( XtScreen (button),
                                        "xlogo32", fg, bg);
                pixmap_reversed = XmGetPixmap ( XtScreen (button),
                                        "xlogo32", bg, fg);
        }
                
        XtVaSetValues (button,
                        XmNlabelPixmap, (XtArgVal) pixmap,
                        NULL);
        pbi -> pixmap = pixmap;
        pbi -> pixmap_reversed = pixmap_reversed;
}

/*************************************************************************
 * Function: PaneButtonCreate ()
 * Purpose: crete a new button in one of the panes
 * In parameters: de, dpi, file_type
 * Function returns: new widget
 * Precondition: 
 * Postcondition: new button created with callbacks set
 *************************************************************************/
Widget
PaneButtonCreate
#ifdef UseFunctionPrototypes
        (char *filename, char *button_name, dir_pane_info *dpi,
	 Widget parent, char file_type)
#else
        (filename, button_name, dpi, parent, file_type)
        char *filename;
        char *button_name;
        dir_pane_info *dpi;
        Widget parent;
        char file_type;

#endif
{
        pane_button_info *pbi = NULL;
        Widget button = NULL;
        static Pixmap label_bug_pixmap = 0; /* illegal value? */
	XGCValues values;

        /* set information needed for this button */
        pbi = (pane_button_info *)
                XtCalloc (1, sizeof (pane_button_info));
        pbi -> name = XtNewString (filename);
        pbi -> dpi = dpi;
        pbi -> file_type = file_type;
        pbi -> pixmap = XmUNSPECIFIED_PIXMAP;
        pbi -> pixmap_reversed = XmUNSPECIFIED_PIXMAP;
        pbi -> gc = gc;
        pbi -> gc_reversed = gc_reversed;

#ifdef DEBUG_APP                                                                
        fprintf (stderr, "Creating button %s\n", filename);
#endif
        /* if recompute_size == True, and we dont have a pixmap
           set at create time, then it always takes its size as 0x0.
           This is (at least) a Motif 1.1.0 fix, that sets a dummy one
        */
        if (label_bug_pixmap == 0)
        {       Pixel fg, bg;

                XtVaGetValues (parent,
                                XmNforeground, (XtArgVal) &fg,
                                XmNbackground, (XtArgVal) &bg,
                                NULL);
                label_bug_pixmap = XmGetPixmap (XtScreen (parent),
                                        "xlogo32", bg, fg);
        }

        button = XmVaCreateDrawnButton (parent,
                                button_name,
                                XmNuserData, (XtArgVal) pbi,
                                XmNlabelType, (XtArgVal) XmPIXMAP,
                                XmNlabelPixmap, (XtArgVal) label_bug_pixmap,
                                XmNmultiClick, (XtArgVal) XmMULTICLICK_KEEP,
                                XmNrecomputeSize, (XtArgVal) False,
#ifdef RDD
                                XmNtranslations, (XtArgVal) trans_table,
#endif /* RDD */
#ifdef MOTIF_DD
                                XmNtranslations, (XtArgVal) trans_table,
#endif /* MOTIF_DD */
                                NULL);

	/* get the appropriate gc's for this button */
        XtVaGetValues (button,
                                XmNforeground, (XtArgVal) &(values.foreground),
                                XmNbackground, (XtArgVal) &(values.background),
                                NULL);
        pbi -> gc = XtGetGC (button, (GCForeground | GCBackground), &values);

        XtVaGetValues (button,
                                XmNforeground, (XtArgVal) &(values.background),
                                XmNbackground, (XtArgVal) &(values.foreground),
                                NULL);
        pbi -> gc_reversed =
		XtGetGC (button, (GCForeground | GCBackground), &values);

	/* and set callbacks */
        XtAddCallback (button, XmNarmCallback,
                                PaneButtonPressed,
                                NULL);
        XtAddCallback (button, XmNactivateCallback,
                                PaneButtonReleased,
                                NULL);
        XtAddCallback (button, XmNexposeCallback,
                                PaneButtonExposed,
                                NULL);
        XtManageChild (button);

        PaneButtonSetPixmap (button, pbi -> name, file_type);

        return button;
}

/*************************************************************************
 * Function: PaneButtonResetLabel ()
 * Purpose: change the label in a widget
 * In parameters: w, label
 * Out parameters: 
 * Precondition: w is a child of Label.
 * Postcondition: w has label set
 *************************************************************************/
void
PaneButtonResetLabel 
#ifdef UseFunctionPrototypes
        (Widget w, char *label, char file_type)
#else
        (w, label, file_type)
        Widget w;
        char * label;
        char file_type;

#endif
{       XmString xmstr = NULL;
        pane_button_info *pbi = NULL;

        /* reset the label in the button */
        xmstr = XmStringCreateSimple (label);
        XtVaSetValues (w,
                        XmNlabelString, (XtArgVal) xmstr,
                        NULL);
        XmStringFree (xmstr);

        /* reset the name in the button info */
        XtVaGetValues (w, XmNuserData, &pbi, NULL);
        XtFree (pbi -> name);
        pbi -> name = XtNewString (label);

        PaneButtonSetPixmap (w, label, file_type);
}

