/*************************************************************************
 * File: $Source$
 * Author: Jan Newmarch
 * Last modified: $Date$
 * Version: $Revision$
 * Purpose: implements the Panel object that contains the three panes of
 *	PaneButton objects
 *
 * Revision history:
 ************************************************************************/ 

/*************************************************************************
 * System includes
 ************************************************************************/ 
#include <X11/Intrinsic.h>
#include <Xm/ScrolledW.h>

/*************************************************************************
 * Local includes
 ************************************************************************/ 
#include "DirMgr.h"
#include "Panel.h"
#include "Pane.h"
#include "types.h"
#include "xmvararg.h"
#include "debug_malloc.h"

/*************************************************************************
 * Functions exported
 ************************************************************************/ 

/*************************************************************************
 * Variables exported
 ************************************************************************/ 
XtTranslations configure_trans_table;

/*************************************************************************
 * Extern variables
 ************************************************************************/ 

/*************************************************************************
 * Extern functions
 ************************************************************************/ 
extern DirectoryMgr *GetFilesInDir (
#ifdef UseFunctionPrototypes
        void
#endif
);


/*************************************************************************
 * Forward functions
 ************************************************************************/ 

/*************************************************************************
 * Local variables
 ************************************************************************/ 

/*************************************************************************
 * Function:
 * Purpose         :
 * In parameters   :
 * Out parameters  :
 * Side effects    :
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 


/*************************************************************************
 * Function: PanelSetFiltersInLabels ()
 * Purpose: set the filters in the three labels
 * In parameters:
 * Out parameters:
 * Precondition;
 * Postcondition:
 ************************************************************************/ 
void
PanelSetFiltersInLabels 
#ifdef UseFunctionPrototypes
	(dir_pane_info *dpi)
#else
	(dpi)
        dir_pane_info *dpi;

#endif
{       char filter[2048];
        XmString xmfilter = NULL;

        strcpy (filter, "Files - filter is ");
        strcat (filter, dpi -> file_filter);
        xmfilter = XmStringCreateSimple (filter);
        XtVaSetValues (dpi -> file_filter_label,
                        XmNlabelString, xmfilter,
                        NULL);
        XmStringFree (xmfilter);

        strcpy (filter, "Executables - filter is ");
        strcat (filter, dpi -> executable_filter);
        xmfilter = XmStringCreateSimple (filter);
        XtVaSetValues (dpi -> executable_filter_label,
                        XmNlabelString, xmfilter,
                        NULL);
        XmStringFree (xmfilter);

        strcpy (filter, "Directories - filter is ");
        strcat (filter, dpi -> dir_filter);
        xmfilter = XmStringCreateSimple (filter);
        XtVaSetValues (dpi -> dir_filter_label,
                        XmNlabelString, xmfilter,
                        NULL);
        XmStringFree (xmfilter);
}

/*************************************************************************
 * Function: PanelCreate ()
 * Purpose: create files, directories display area
 * In parameters: form, dpi
 * Function result: pane widget
 * Side effects; fields in dpi point to 3 pane children
 * Precondition: form is non-null
 * Postcondition: pane created with children in this geometric relation
 *                --------------
 *               | executables  |
 *               | pane         |
 *                --------------
 *               | files pane   |
 *                --------------
 *               | dirs pane    |
 *                --------------
 ************************************************************************/ 
Widget
PanelCreate 
#ifdef UseFunctionPrototypes
	(Widget form, dir_pane_info *dpi)
#else
	(form, dpi)
	Widget form;
	dir_pane_info *dpi;

#endif
{
	Widget	pane = NULL,
		dirs_pane = NULL,
		files_pane = NULL,
		executables_pane = NULL,
		rc1 = NULL, rc2 = NULL, rc3 = NULL,
		file_filter_label = NULL,
		dir_filter_label = NULL,
		executable_filter_label = NULL,
		sw1 = NULL, sw2 = NULL, sw3 = NULL;
	XtActionsRec actions_rec;

#ifdef ALLOW_ROWCOL_RESIZE
	/* the action called by this is to allow the number of columns
	   to be dynamically reset when the enclosing pane is resized.
	   It throws up a RowCol bug under Motif 1.1.0
	*/
	/* we need to spot when the user horizontally resizes panes
	   to reset number of columns of files shown */
        actions_rec.string = "resize";
        actions_rec.proc = PaneResized;

	XtAddActions (&actions_rec, 1);
	configure_trans_table = XtParseTranslationTable ("<Configure>: resize()");
#endif /* ALLOW_ROWCOL_RESIZE */

#ifdef DEBUG_APP                                                                      
        fprintf (stderr, "Creating paned window for buttons\n");
#endif
	pane = XmVaCreatePanedWindow (form, "pane",
			NULL);

	/* create the executable files section */
	/* a row column of a label and a row column of drawn buttons */

#ifdef DEBUG_APP                                                                      
        fprintf (stderr, "Creating pane for executables\n");
#endif
	rc1 = PaneCreate (pane, &(dpi -> executables_pane), "executables_pane",
		&(dpi -> executable_filter_label), "executable_filter_label", dpi);

	/* create the ordinary files section */
	/* a row column of a label and a row column of drawn buttons */

#ifdef DEBUG_APP                                                                      
        fprintf (stderr, "Creating pane for files\n");
#endif

	rc2 = PaneCreate (pane, &(dpi -> files_pane), "files_pane",
		&(dpi -> file_filter_label), "file_filter_label", dpi);

	/* create the directories section */
	/* a row column of a label and a row column (in a scrolled window)
	   of drawn buttons */

#ifdef DEBUG_APP                                                                      
        fprintf (stderr, "Creating pane for directories\n");
#endif

	rc3 = PaneCreate (pane, &(dpi -> dirs_pane), "dirs_pane",
		&(dpi -> dir_filter_label), "dir_filter_label", dpi);

	return pane;
}



/*************************************************************************
 * Function: PanelSetFiles ()
 * Purpose: Set up the file names in buttons in each of the panes
 * In parameters:  dpi
 * Out parameters: dpi contents changed by indirection
 * Precondition: directory list in dm, 3 non-null pane widgets,
 *               dpi points to existing structure
 * Postcondition: some fields in dpi filled in, panes populated with
 *                buttons of file names
 ************************************************************************/ 
void
PanelSetFiles 
#ifdef UseFunctionPrototypes
	(dir_pane_info *dpi)
#else
	(dpi)
	dir_pane_info *dpi;

#endif
{
	DirectoryMgr *dm = NULL;
        DirEntry *de = NULL;
	int dirs_max, executables_max, files_max;
	Widget *pbutton;

	dm = dpi -> directory_manager;

	SetDirectoryLabel (dpi -> directory_label, dm);

	DirPaneUnshow (dpi);
	ExecutablePaneUnshow (dpi);
	FilePaneUnshow (dpi);

	dirs_max	= dpi -> dirs_buttons_max;
	executables_max	= dpi -> executables_buttons_max;
	files_max	= dpi -> files_buttons_max;

        /* ensure we are at the beginning of the directory list */
        DirectoryMgrRestart (dm);
	dpi -> dirs_buttons_last = -1;
	dpi -> executables_buttons_last = -1;
	dpi -> files_buttons_last = -1;

	while ( (de = DirectoryMgrNextEntry(dm)) != NULL)
        {
		/* explanation of code:
		   for each file in list, it is tested to see if it
		   is a directory, an executable or other file.
		   Once classified: if there is no room in the array
		   use XtRealloc to increase array size. If the next
		   spare location is empty (NULL), create a new button
		   for it, otherwise reuse the button by resetting its
		   label.
		*/

		/* place dirs in dir pane */
		if (DirEntryIsDir (de) ||
			DirEntryIsSymLink (de) &&
				S_ISDIR (FileInfoOrigMode (DirEntryActualInfo (de))))
		{
			/* does it match pattern? else discard it */
			if ( !RegExpMatch (DirEntryFileName (de),
					dpi -> dir_filter_regexp))
				continue;

			DirPaneAddFile (DirEntryFileName (de), dpi);
		}
		/* and executables in executables pane */
		else if ((FileInfoOrigMode( DirEntryActualInfo (de))
				& (S_IXUSR | S_IXOTH | S_IXGRP)) != 0)
		{
			/* does it match pattern? else discard it */
			if ( !RegExpMatch (DirEntryFileName (de),
					dpi -> executable_filter_regexp))
				continue;

                        ExecutablePaneAddFile (DirEntryFileName (de), dpi);
		}
		/* everything else in files pane */
		else
		/* does it match pattern? else discard it */
		if (RegExpMatch (DirEntryFileName (de),
				dpi -> file_filter_regexp))
		{
                        FilePaneAddFile (DirEntryFileName (de), dpi);
		}
        }


	DirPaneShow (dpi);
	ExecutablePaneShow (dpi);
	FilePaneShow (dpi);
}



/*************************************************************************
 * Function: FindFileInPane()
 * Purpose: given file name, find the widget button with that name
 * In parameters: file_name, files
 * Out parameters: button
 * Precondition:
 * Postcondition: button points to widget with file_name
 ************************************************************************/ 
static void FindFileInPane 
#ifdef UseFunctionPrototypes
	(char *file_name, WidgetList files, int num_files, Widget *button)
#else
	(file_name, files, num_files, button)
	char *file_name;
	WidgetList files;
	int num_files;
	Widget *button;

#endif
{	pane_button_info *pbi = NULL;
	int i;

	/* traverse current list. list ends on NULL
	   or when we hit an unmanaged widget
	*/
	for (i = 0; i < num_files; i++)
	{	XtVaGetValues (files[i], 
				XmNuserData, &pbi,
				NULL);
#ifdef DEBUG
                fprintf (stderr, "find file: checking %s against %s\n",
                           file_name, pbi -> name);
#endif
		if (strcmp (file_name, pbi -> name) == 0)
		{	/* found button */
			*button = files[i];
			return;
		}
	}
	*button = NULL;
}

/*************************************************************************
 * Function: MoveScrolledWindowToFile ()
 * Purpose: set the scrolled window so that this file shows selected in it
 * In parameters: fle_name, dpi
 * Out parameters:
 * Precondition:
 * Postcondition:
 ************************************************************************/ 
static void 
MoveScrolledWindowToFile 
#ifdef UseFunctionPrototypes
	(char *file_name, dir_pane_info *dpi)
#else
	(file_name, dpi)
	char *file_name;
	dir_pane_info *dpi;

#endif
{
	Widget file_button = NULL;
	Widget vert_sb = NULL;
	Widget w = NULL;
	Position x, y;
	Dimension width, height;
	int max, min, value;
	int slider, incr, page;

	if (file_name == NULL)
		return;

	/* deselect old file */
	if (dpi -> file_selected != NULL)
	{
		PaneButtonInvertColours (dpi -> file_button_selected);
		ToolbarClear (dpi -> files_toolbar);
	}
	/* find the file_name in one of the three panes */
	file_button = NULL;
	FindFileInPane (file_name, dpi -> dirs_buttons,
			dpi -> dirs_buttons_last + 1, &file_button);
	if (file_button == NULL)
		FindFileInPane (file_name, dpi -> files_buttons,
			dpi -> files_buttons_last + 1, &file_button);
	if (file_button == NULL)
		FindFileInPane (file_name, dpi -> executables_buttons,
			dpi -> executables_buttons_last + 1, &file_button);
	if (file_button == NULL)
		/* couldn't find it anywhere */
		return;
	
	/* select button, and set actions (a bit kludgy) */
	PaneButtonPressed (file_button, NULL, NULL);

	/* where is it in row col? */
	XtVaGetValues (file_button,
		XmNx, &x, XmNy, &y, NULL);
	XtVaGetValues (XtParent (file_button),
		XmNwidth, &width,
		XmNheight, &height,
		NULL);

	/* find the vertical slider */
	w = XtParent (file_button);
	while ( !XtIsSubclass (w, xmScrolledWindowWidgetClass))
		w = XtParent (w);
	XtVaGetValues (w,
			XmNverticalScrollBar, &vert_sb,
			NULL);
	/* set the vertical slider to proportion of rowcol*/
	if (vert_sb == NULL)
		return;
	XtVaGetValues (vert_sb,
		XmNminimum, &min,
		XmNmaximum, &max,
		NULL);
	XmScrollBarGetValues (vert_sb, &value, &slider,
		&incr, &page);
	value = min + ((max - min) * y) / height;

	/* make sure we leave space for slider */
	if (value > max - slider)
		value = max - slider;

	XmScrollBarSetValues (vert_sb, value, slider,
		incr, page, True);
}

/*************************************************************************
 * Function: ResetPanesAndSelectFile ()
 * Purpose: update pane (if needed) to show file changes, and
 * 	select new file (if possible)
 * In parameters: file_name, dpi
 * Out parameters:
 * Precondition:
 * Postcondition:
 ************************************************************************/ 
void
ResetPanesAndSelectFile 
#ifdef UseFunctionPrototypes
	(dir_pane_info *dpi, char *orig_file_name)
#else
	(dpi, orig_file_name)
	dir_pane_info *dpi;
	char *orig_file_name;

#endif
{	DirectoryMgr *dm = NULL;
        char *file_name = NULL;

        /* orig_file may be pointer into a structure that gets massacred
           by DirectoryMgrClose. Avoid dangling pointer by copying first
        */
	if (orig_file_name != NULL)
        {	file_name = XtMalloc (strlen (orig_file_name) + 1);
        	strcpy (file_name, orig_file_name);
	}
	else file_name = NULL;

	/* now reset panes and select this file ... */
	/* should only do this if (a) new worked, and (b) in current dir */

	if (dpi -> file_selected != NULL)
	{	PaneButtonInvertColours (dpi -> file_button_selected);
		dpi -> file_selected = NULL;
		ToolbarClear (dpi -> files_toolbar);
	}

        DirectoryMgrClose (dpi -> directory_manager);
        dm = GetFilesInDir ();
        dpi -> directory_manager = dm;
	PanelSetFiles (dpi);
	MoveScrolledWindowToFile (file_name, dpi);

        if (file_name != NULL) XtFree (file_name);
}
