/*************************************************************************
 * File: $Source$
 * Author: Jan Newmarch
 * Last modified: $Date$
 * Version: $Revision$
 * Purpose:
 *
 * Revision history:
 *	 2 Apr 93	created from applicat.c
 ************************************************************************/ 

/*************************************************************************
 * System includes
 ************************************************************************/ 
#include <X11/Intrinsic.h>

/*************************************************************************
 * Local includes
 ************************************************************************/ 
#include "const.h"
#include "types.h"
#include "xmvararg.h"
#include "ToolbarB.h"

#ifdef RDD
#include "rdd.h"
#endif

/*************************************************************************
 * Functions exported
 ************************************************************************/ 

/*************************************************************************
 * Variables exported
 ************************************************************************/ 

/*************************************************************************
 * Extern variables
 ************************************************************************/ 
extern XmString empty_string;

/*************************************************************************
 * Extern functions
 ************************************************************************/ 

/*************************************************************************
 * Forward functions
 ************************************************************************/ 

/*************************************************************************
 * Local variables
 ************************************************************************/ 

/*************************************************************************
 * Function:
 * Purpose         :
 * In parameters   :
 * Out parameters  :
 * Side effects    :
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 



/*************************************************************************
 * Function: FilesToolbarCreate ()
 * Purpose: create the files toolbar and populate it with toolbar buttons
 * In parameters: form, dpi
 * Function result: files toolbar widget
 * Precondition: form is non-null
 * Postcondition: toolbar + buttons created, each button has dpi in user data
 ************************************************************************/ 
Widget
FilesToolbarCreate 
#ifdef UseFunctionPrototypes
	(Widget form, dir_pane_info *dpi)
#else
	(form, dpi)
	Widget form;
	dir_pane_info *dpi;

#endif
{
	Widget	button = NULL,
		files_toolbar = NULL,
		*files_toolbar_buttons = NULL;
	int 	i;
	tool_button_info *tbi = NULL;
		
	files_toolbar_buttons = (Widget *) XtCalloc (1, sizeof (Widget) *
						FILES_TOOLBAR_SIZE);
	dpi -> files_toolbar = files_toolbar_buttons;

	files_toolbar = XmVaCreateRowColumn (form, "files_toolbar",
			XmNpacking,		(XtArgVal) XmPACK_COLUMN,
			XmNorientation,		(XtArgVal) XmHORIZONTAL,
#ifdef SIZE_KLUDGE
			/* set to 4 when we only show 3 is because of
			   dummy widget in 4th row */
			XmNnumColumns, 		(XtArgVal) 4,
#else
                        XmNnumColumns,          (XtArgVal) 3,
#endif /*SIZE_KLUDGE*/
			XmNadjustLast,		(XtArgVal) False,
			NULL);
	XtManageChild (files_toolbar);

	for (i = 0; i < FILES_TOOLBAR_SIZE; i++)
		files_toolbar_buttons[i] = ToolbarButtonCreate (files_toolbar,
						"file_button", dpi);

#ifdef SIZE_KLUDGE
	{	/* kludge to force the row column to give
		   us the size buttons we want.  It doesn't
		   tamper with the size of DrawButtons
		*/

	 	Widget dummy;
		dummy = XmVaCreateDrawnButton (files_toolbar, "dummy",
				XmNmappedWhenManaged, (XtArgVal) False,
				NULL);
		XtManageChild (dummy);
	}
#endif /*SIZE_KLUDGE */

	return files_toolbar;
}

/*************************************************************************
 * Function: ToolbarClear ()
 * Purpose: empty out the files toolbar of any actions
 * In parameters: toolbar
 * Function result: 
 * Precondition: 
 * Postcondition: all toolbar buttons show empty string
 ************************************************************************/ 
void
ToolbarClear
#ifdef UseFunctionPrototypes
	(Widget *toolbar)
#else
	(toolbar)
	Widget *toolbar;

#endif
{
	int i;
	
	for (i = 0; i < FILES_TOOLBAR_SIZE; i++)
	{
		if ( !XtIsSensitive (*toolbar))
			return;	
		XtVaSetValues (*toolbar,
				XmNlabelString, empty_string,
				NULL);
		XtSetSensitive (*toolbar, False);
		toolbar++;
	}
}

/*************************************************************************
 * Function: DirsToolbarCreate ()
 * Purpose: create the dirs toolbar and populate it with toolbar buttons
 * In parameters: form, dpi
 * Function result: dirs toolbar widget
 * Precondition: form is non-null
 * Postcondition: toolbar + buttons created, each button has dpi in user data
 ************************************************************************/ 
Widget
DirsToolbarCreate 
#ifdef UseFunctionPrototypes
	(Widget form, dir_pane_info *dpi)
#else
	(form, dpi)
	Widget form;
	dir_pane_info *dpi;

#endif
{
	Widget	button = NULL,
		dirs_toolbar = NULL,
		*dirs_toolbar_buttons = NULL;
	int	i;
	tool_button_info *tbi = NULL;

	dirs_toolbar_buttons = (Widget *) XtCalloc (1, sizeof (Widget) *
						DIRS_TOOLBAR_SIZE);
	dpi -> dirs_toolbar = dirs_toolbar_buttons;

	dirs_toolbar = XmVaCreateRowColumn (form, "dirs_toolbar",
			XmNpacking,		(XtArgVal) XmPACK_COLUMN,
			XmNorientation,		(XtArgVal) XmHORIZONTAL,
#ifdef SIZE_KLUDGE
			/* set to 4 when we only show 3 is because of
			   dummy widget in 4th row */
			XmNnumColumns, 		(XtArgVal) 4,
#else
                        XmNnumColumns,          (XtArgVal) 3,
#endif /*SIZE_KLUDGE*/
                        XmNadjustLast,           (XtArgVal) False,
			NULL);
	XtManageChild (dirs_toolbar);

	for (i = 0; i < DIRS_TOOLBAR_SIZE; i++)
		dirs_toolbar_buttons[i] = ToolbarButtonCreate (dirs_toolbar,
						"dir_button", dpi);

#ifdef SIZE_KLUDGE
	{	/* kludge to force the row column to give
		   us the size buttons we want.  It doesn't
		   tamper with the size of DrawButtons
		*/
	 	Widget dummy;
		dummy = XmVaCreateDrawnButton (dirs_toolbar, "dummy",
				XmNmappedWhenManaged, (XtArgVal) False,
				NULL);
		XtManageChild (dummy);
	}
#endif /*SIZE_KLUDGE*/

	return dirs_toolbar;
}

void
ToolbarSetActions
#ifdef UseFunctionPrototypes
	(WidgetList toolbar, file_action *pfa)
#else
	(toolbar, pfa)
	WidgetList toolbar;
	file_action *pfa;
#endif
{
	int button;
	XmString xmstr;
	tool_button_info *tbi;
	action_pair *pap;
	Widget tool_button;

        button = 0;
        /* set actions in toolbar */
        if (pfa != NULL)
        {
                pap = pfa -> actions;
                while (button < FILES_TOOLBAR_SIZE && pap != NULL)
                {
                        tool_button = toolbar [button];

                        /* set the action label in the button */
                        xmstr = XmStringCreateLtoR (pap -> action_label,
                                        XmSTRING_DEFAULT_CHARSET);
                        XtVaSetValues (tool_button,
                                        XmNlabelString, (XtArgVal) xmstr,
                                        NULL);
                        XmStringFree (xmstr);

                        /* set the action in the buttons user data */
                        XtVaGetValues (tool_button,
                                        XmNuserData, (XtArgVal) &tbi,
                                        NULL);
                        tbi -> action = pap -> action;
                        tbi -> prompt = pap -> prompt;
                        tbi -> run_in_xterm = pap -> run_in_xterm;
                        tbi -> pause_after_exec = pap -> pause_after_exec;
                        tbi -> has_prompt = pap -> has_prompt;
                        tbi -> confirm = pap -> confirm;
                        tbi -> has_confirm = pap -> has_confirm;

                        XtSetSensitive (tool_button, True);
                        button++;
                        pap = pap -> next;
                }
        }
        /* empty out the remaining buttons */
        for ( ; button < FILES_TOOLBAR_SIZE; button++)
        {
                tool_button = toolbar [button];
                /* break out as soon as we hit an already insensitive button */
                if (XtIsSensitive (tool_button) == False)
                        break;
                XtVaSetValues (tool_button,
                                XmNlabelString, (XtArgVal) empty_string,
                                NULL);
                XtSetSensitive (tool_button, False);
        }
}

