/*************************************************************************
 * File: $Source$
 * Author: Jan Newmarch
 * Last modified: $Date$
 * Version: $Revision$
 * Purpose:
 *
 * Revision history:
 *	 2 Apr 93	created from applicat.c
 ************************************************************************/ 

/*************************************************************************
 * System includes
 ************************************************************************/ 

/*************************************************************************
 * Local includes
 ************************************************************************/ 
#include "const.h"
#include "types.h"
#include "xmvararg.h"
#include "Prompt.h"
#include "Confirm.h"

#ifdef RDD
#include "rdd.h"
#endif

/*************************************************************************
 * Functions exported
 ************************************************************************/ 

/*************************************************************************
 * Variables exported
 ************************************************************************/ 

/*************************************************************************
 * Extern variables
 ************************************************************************/ 
extern XmString empty_string;

/*************************************************************************
 * Extern functions
 ************************************************************************/ 
extern void
ToolbarDrop (
#ifdef UseFunctionPrototypes
        Widget w, XtPointer call, XtPointer client_data
#endif
);


/*************************************************************************
 * Forward functions
 ************************************************************************/ 

/*************************************************************************
 * Local variables
 ************************************************************************/ 
#define IS_WHITE_SPACE(c)       ((c) == ' ' || (c) == '\t' || (c) == '\n')

#define IS_BUILTIN_COMMAND(s)	(strncmp (s, CHDIR_COMMAND, LEN_CHDIR_COMMAND) == 0 || \
				 strncmp (s, OPENDIR_COMMAND, LEN_OPENDIR_COMMAND) == 0 || \
				 strncmp (s, INFO_COMMAND, LEN_INFO_COMMAND) == 0)


/*************************************************************************
 * Function:
 * Purpose         :
 * In parameters   :
 * Out parameters  :
 * Side effects    :
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 


/*************************************************************************
 * Function: ToolbarButtonPressed ()
 * Purpose: handle a button press in the file toolbar
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition: button in toolbar pressed
 * Postcondition: action for that button on selected file executed
 ************************************************************************/ 
void
ToolbarButtonPressed
#ifdef UseFunctionPrototypes
        (Widget w, XtPointer client_data, XtPointer call_data)
#else
        (w, client_data, call_data)
        Widget w;
        XtPointer client_data, call_data;

#endif
{
        tool_button_info *tbi = NULL;
        char *command = NULL;
        char *argv[10];
        int argc;
        char *prompt_answer = NULL;
        int len_arg;

        XtVaGetValues (w,
                        XmNuserData, (XtArgVal) &tbi,
                        NULL);

        argv[0] = tbi -> dpi -> file_selected;
        if (argv[0] != NULL)
                argc = 1;
        else    argc = 0;
        if (tbi -> has_prompt)
        {       PromptDialog (w, tbi -> prompt, True, &prompt_answer);
                if (prompt_answer == NULL)
                        return;
                /* set answer into args array */
                while (*prompt_answer != '\0' && argc < 10)
                {       while (IS_WHITE_SPACE (*prompt_answer))
                                prompt_answer++;
                        argv[argc] = prompt_answer;
                        argc++;
                        len_arg = strcspn (prompt_answer, " \t\n");
                        if (prompt_answer[len_arg] == '\0')
                                break;
                        else
                        {       prompt_answer[len_arg] = '\0';
                                prompt_answer += len_arg + 1;
                        }
                }
        }
#ifdef DEBUG
        for (len_arg = 0; len_arg < argc; len_arg++)
                fprintf (stderr, "Arg %d is %s\n", len_arg, argv[len_arg]);
#endif

        if (tbi -> has_confirm)
        {       /* should we proceed with this action? */
                char *confirm;
                Bool confirm_answer;

#ifdef DEBUG
                fprintf (stderr, "confirming action...\n");
#endif
                expand_command (&confirm, argv, argc, tbi -> confirm);
                ConfirmDialog (w, confirm, True, &confirm_answer);
#ifdef DEBUG
                if (confirm_answer == OK_ANSWER)
                        fprintf (stderr, "confirmed\n");
                else    fprintf (stderr, "cancelled\n");
#endif
                if (confirm_answer != OK_ANSWER)
                        return;
        }

        expand_command (&command, argv, argc, tbi -> action);

#ifdef DEBUG
        fprintf (stderr, "action %s\n", tbi -> action);
        fprintf (stderr, "on file %s\n", tbi -> dpi -> file_selected);
        fprintf (stderr, "new command: %s\n", command);
#endif

        /* make sure we are in the right directory for this pane */
        XtChdir (DirectoryPath (DirectoryMgrDir (tbi -> dpi -> directory_manager)),
                        tbi -> dpi -> toplevel);

#ifdef DEBUG
        {       char path[MAXPATHLEN + 2];
                fprintf (stderr, "changing to directory %s\n",
                        DirectoryPath (DirectoryMgrDir (tbi -> dpi -> directory_

                GetCWD (path, MAXPATHLEN);
                fprintf (stderr, "now in directory %s\n", path);
        }
#endif

        if ( IS_BUILTIN_COMMAND (command+1))
                builtin_command (command, tbi -> dpi);
        else
                do_command (command, tbi -> run_in_xterm, 
                                     tbi -> pause_after_exec,
                                     tbi -> dpi);
        XtFree (command);
}

/*************************************************************************
 * Function	   : ToolbarButtonCreate
 * Purpose         : create a new button for the file action toolbar 
 * In parameters   : parent, name
 * Out parameters  :
 * Side effects    :
 * Function returns:
 * Precondition    :
 * Postcondition   :
 ************************************************************************/ 
Widget
ToolbarButtonCreate
#if UseFunctionPrototypes
	(Widget parent, char *name, dir_pane_info *dpi)
#else
	(parent, name, dpi)
	Widget parent;
	char *name;
	dir_pane_info *dpi;
#endif
{
	Widget button;
	tool_button_info *tbi;

        tbi = (tool_button_info *) XtCalloc (1, sizeof (tool_button_info));
        tbi -> dpi = dpi;

        button =
                XmVaCreatePushButton (parent, name,
                        XmNlabelString, (XtArgVal) empty_string,
                        XmNuserData, (XtArgVal) tbi,
                        NULL);
        XtAddCallback (button,
                XmNactivateCallback, ToolbarButtonPressed,
                NULL);
        XtManageChild (button);
        XtSetSensitive (button, False);

#ifdef RDD
        rddAddDropHandler (button,
                        ToolbarDrop, NULL);
#endif /* RDD */

#ifdef MOTIF_DD
        MotifDDRegisterToolbar (button);
#endif /* MOTIF_DD */

	return button;
}


