/*************************************************************************
 * File: $Source: /usr/usrs/xsource/xmfm/RCS/applicat.c,v $
 * Author: Jan Newmarch
 * Last modified: $Date: 1993/04/01 04:15:19 $
 * Version: $Revision: 1.14 $
 * Purpose: Create the application windows and set up callbacks
 *
 * Revision history:
 *	4 Aug 92	added filter stuff to ResetFilesInPanes
 *	4 Aug 92	added R5 hooks to allow editres to look at this applic
 *	5 Aug 92	put Toolbars in scrolled windows, 
 *			to stop clipping on resize
 *	5 Aug 92	set dir label above panes only, instead of across form
 *	6 Aug 92	removed scrolled windows from Toolbars,
 *			reset dir label as before. let left edge of
 *			toolbars be free
 *	16 Oct 92	caddr_t changed to XtPointer
 *       3 Nov 92       lint-ed
 *	11 Nov 92	follow symbolic links to show dirs as dirs, not files.
 *			Moved Rogers drag 'n drop to drag.c
 *      26 Nov 92       Added PIXMAPDIR
 *	30 Nov 92	used GetCWD
 *	25 Mar 93	all pointer vbls grounded on definition, many
 *			XtMallocs changed to XtCalloc to trap pointer bugs
 *	 1 Apr 93	dynamic pane button alloc changed to array
 *	 3 Apr 93	lots of stuff moved out to PaneB.c, Pane.c, Panel.c
 ************************************************************************/ 

#include "copyright.h"

/*************************************************************************
 * System includes
 ************************************************************************/ 
#include <sys/types.h>
#include <sys/stat.h>
#include <Xm/MainW.h>
#include <stdlib.h>
#include <Xm/PushB.h>
#include <X11/Xlib.h>

/*************************************************************************
 * Local includes
 ************************************************************************/ 
#include "xmvararg.h"
#include "DirMgr.h"
#include "const.h"
#include "types.h"
#include "RegExp.h"
#include "debug_malloc.h"
#include "Pane.h"
#include "Panel.h"
#include "Toolbar.h"
#include "ToolbarB.h"

#ifdef RDD
#include "rdd.h"
#endif /* RDD */

/*************************************************************************
 * Functions exported
 ************************************************************************/ 
extern Widget CreateApplication (
#ifdef UseFunctionPrototypes
	Widget parent
#endif
);

/*************************************************************************
 * Variables exported
 ************************************************************************/ 

/*************************************************************************
 * Extern functions
 ************************************************************************/ 
extern Widget CreateMenu (
#ifdef UseFunctionPrototypes
	Widget parent, XtPointer user_data
#endif
);

extern char *GetCWD (
#ifdef UseFunctionPrototypes
        char *, int
#endif
);

#ifdef ALLOW_EDITRES
extern void (* _XEditResCheckMessages) (
#if UseFunctionPrototypes
	Widget w, XtPointer data, XEvent *event, Boolean *
#endif
);
#endif /* ALLOW_EDITRES */

#ifdef RDD
extern void MainWindowDrop (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer call, XtPointer client_data
#endif
);
#endif /* RDD */

/*************************************************************************
 * Extern variables
 ************************************************************************/ 


/*************************************************************************
 * Forward functions
 ************************************************************************/ 
Widget CreateApplication (
#ifdef UseFunctionPrototypes
	Widget parent
#endif
);	

DirectoryMgr *GetFilesInDir (
#ifdef UseFunctionPrototypes
	void
#endif
);


/*************************************************************************
 * Local variables
 ************************************************************************/ 
XmString empty_string = NULL;

/*************************************************************************
 * Function: SetGeometry ()
 * Purpose: set the geometry of components in the form
 * In parameters: directory_label, files_toolbar, dirs_toobar, pane
 * Function result:
 * Precondition: all args are non-null children of a form
 * Postcondition: all args are set in the following geometry
 *		--------------------------
 *             |               dir label  |
 *             |--------------------------|
 *             | files       |            |
 *             | toolbar     |  pane      |
 *             |-------------|            |
 *             | dirs        |            |
 *             | toolbar     |            |
 *              --------------------------
 ************************************************************************/ 
static void
SetGeometry 
#ifdef UseFunctionPrototypes
	(Widget directory_label, Widget files_toolbar, Widget dirs_toolbar, Widget pane)
#else
	(directory_label, files_toolbar, dirs_toolbar, pane)
	Widget directory_label;
	Widget files_toolbar;
	Widget dirs_toolbar;
	Widget pane;

#endif
{
	XtVaSetValues (directory_label,
			XmNtopAttachment,	(XtArgVal) XmATTACH_FORM,
			XmNleftAttachment,	(XtArgVal) XmATTACH_FORM,
			XmNrightAttachment,	(XtArgVal) XmATTACH_FORM,
			NULL);

	XtVaSetValues (files_toolbar,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) directory_label,
			XmNleftAttachment,	(XtArgVal) XmATTACH_FORM,
			XmNbottomAttachment,	(XtArgVal) XmATTACH_POSITION,
			XmNbottomPosition,	(XtArgVal) 45,
			NULL);

	XtVaSetValues (dirs_toolbar,
			XmNtopAttachment,	(XtArgVal) XmATTACH_POSITION,
			XmNtopPosition,		(XtArgVal) 50,
			XmNleftAttachment,	(XtArgVal) XmATTACH_FORM,
                        XmNbottomAttachment,    (XtArgVal) XmATTACH_FORM,
			NULL);

	XtVaSetValues (pane,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) directory_label,
			XmNleftAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNleftWidget,		(XtArgVal) files_toolbar,
			XmNrightAttachment,	(XtArgVal) XmATTACH_FORM,
			XmNbottomAttachment,	(XtArgVal) XmATTACH_FORM,
			NULL);
}

/*************************************************************************
 * Function: CreateApplication ()
 * Purpose: create a toplevel widget
 * In parameters: parent
 * Function result: toplevel widget in managed state
 * Precondition: parent is non-null widget
 * Postcondition: widget hierarchy set up and managed
 ************************************************************************/ 
Widget CreateApplication 
#ifdef UseFunctionPrototypes
	(Widget parent)
#else
	(parent)  
Widget		parent;		/*  parent widget	*/ 

#endif
{ 
	Widget		main = NULL,
			form = NULL,
			directory_label = NULL,
			pane = NULL,
			menu = NULL,
			dirs_toolbar = NULL,
			files_toolbar = NULL;
 
	char	str[128];
	DirectoryMgr *files = NULL;
	dir_pane_info *dpi = NULL;

#ifdef ALLOW_EDITRES
	XtAddEventHandler (parent, (EventMask) 0, True,
				_XEditResCheckMessages, NULL);
#endif

#ifdef DEBUG_APP                                                                      
        fprintf (stderr, "Creating application panes\n");
#endif
                                                                                
	empty_string = XmStringCreateSimple ("           ");

	dpi = (dir_pane_info *) XtCalloc (1, sizeof (dir_pane_info));

	main = XmVaCreateMainWindow (parent, "main", 
			XmNuserData, (XtArgVal) dpi,
			NULL);
#ifdef RDD
/*	seems to override other drops 
	rddAddDropHandler (main,
				MainWindowDrop, NULL);
*/
#endif /* RDD */

	form = XmVaCreateForm (main, "form",
			NULL);


	directory_label = XmVaCreateLabel (form, "directory_label",
					NULL);
#ifdef DEBUG_APP                                                                      
        fprintf (stderr, "Creating files_toolbar\n");
#endif
	files_toolbar = FilesToolbarCreate (form, dpi);

#ifdef DEBUG_APP                                                                      
        fprintf (stderr, "Creating dirs toolbar\n");
#endif
	dirs_toolbar = DirsToolbarCreate (form, dpi);

#ifdef DEBUG_APP                                                                      
        fprintf (stderr, "Creating buttons pane\n");
#endif
	pane = PanelCreate (form, dpi);

	files = GetFilesInDir ();

	/* initialise the pane contents */
	dpi -> dirs_buttons		= NULL;
	dpi -> files_buttons		= NULL;
	dpi -> executables_buttons	= NULL;

	dpi -> dirs_buttons_last	= -1;
	dpi -> files_buttons_last	= -1;
	dpi -> executables_buttons_last	= -1;

	dpi -> dirs_buttons_max		= 0;
	dpi -> files_buttons_max	= 0;
	dpi -> executables_buttons_max	= 0;

#ifdef DEBUG_APP                                                                      
        fprintf (stderr, "Creating menu\n");
#endif
	menu = CreateMenu (main, (XtPointer) dpi);

	XtManageChild (menu);
	XtManageChild (dirs_toolbar);
	XtManageChild (files_toolbar);
	XtManageChild (pane);
	XtManageChild (directory_label);
	XtManageChild (form);
	XtManageChild (main);

	SetGeometry (directory_label, files_toolbar, dirs_toolbar, pane);

	/* set up the user data stuff */
	dpi -> directory_label = directory_label;
	dpi -> directory_manager = files;
	dpi -> file_selected = NULL;
	dpi -> file_button_selected = NULL;
	dpi -> dir_selected = NULL;
	dpi -> executable_selected = NULL;
	dpi -> toplevel = parent;
        dpi -> run_dialog = NULL;
        dpi -> info_dialog = NULL;

	/* reference counting for refresh.c */
	dpi -> being_destroyed = False;
	dpi -> reference_count = 0;

	XmMainWindowSetAreas (main, menu, NULL, NULL, NULL, form);

	return (main);
} 


/*************************************************************************
 * Function: GetFilesInDir ()
 * Purpose: return a list of files in current directory
 * Function result: list of files
 * Precondition: none
 * Postcondition: list of files returned
 ************************************************************************/ 
DirectoryMgr *GetFilesInDir 
#ifdef UseFunctionPrototypes
	(void)
#else
	()

#endif
{
        DirectoryMgr *dm = NULL;
	char path[MAXPATHLEN + 2];

	GetCWD (path, MAXPATHLEN);

	dm = DirectoryMgrSimpleOpen(path,DIR_MGR_SORT_NAME_DIRS_FIRST, (char *)NULL);
        if (dm == NULL)
        {
                fprintf(stderr,"Can't open directory mgr for '%s'\n",path);
                exit(-1);
        }
	return (dm);
}


/*************************************************************************
 * Function: SetDirectoryLabel ()
 * Purpose: set the current dir path in the Label widget showing above panes
 * In parameters:  dir_label, dm
 * Out parameters: 
 * Precondition: 
 * Postcondition: labelString in dir_label set to current dir
 ************************************************************************/ 
void
SetDirectoryLabel 
#ifdef UseFunctionPrototypes
	(Widget dir_label, DirectoryMgr *dm)
#else
	(dir_label, dm)
	Widget dir_label;
	DirectoryMgr *dm;

#endif
{
	char *dir = NULL;
	XmString xmdir = NULL;
	char full_dir[MAXPATHLEN + 2];

	dir = DirectoryPath (DirectoryMgrDir (dm));
	strcpy (full_dir, "Directory: ");
	strcat (full_dir, dir);
	xmdir = XmStringCreateSimple (full_dir);
	XtVaSetValues (dir_label,
			XmNlabelString, xmdir,
			NULL);
	XmStringFree (xmdir);
}


