/*************************************************************************
 * File: $Source: /usr/usrs/xsource/xmfm/RCS/buttoncb.c,v $
 * Author: Jan Newmarch
 * Last modified: $Date: 1993/04/01 04:15:23 $
 * Version: $Revision: 1.13 $
 * Purpose: handle the callback events from various buttons being pressed
 *
 * Revision history:
 *	7 Aug 92	stdin of no-xterm do_commands set to /dev/null
 *	16 Oct 92	caddr_t changed to XtPointer
 *       3 Nov 92       lint-ed
 *	30 Nov 92	used GetCWD
 *	 5 Dec 92	looks up SHELL in environment
 *	21 Dec 92	compiled under g++
 *	25 Mar 93	all pointer vbls grounded on definition, many
 *			XtMallocs changed to XtCalloc to trap pointer bugs
 *	26 Marc 93	command array lengthened to stop overflow
 ************************************************************************/ 

#include "copyright.h"

/*************************************************************************
 * System includes
 ************************************************************************/ 
#include <Xm/Xm.h>
#include <X11/cursorfont.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>

/*************************************************************************
 * Local includes
 ************************************************************************/ 
#include "const.h"
#include "types.h"
#include "RegExp.h"
#ifdef RDD
#include "rdd.h"
#endif
#include "debug_malloc.h"

/*************************************************************************
 * Functions exported
 ************************************************************************/ 
extern void do_command (
#ifdef UseFunctionPrototypes
	char *command, int run_in_xterm, int pause_after_exec, dir_pane_info *dpi
#endif
);
extern void add_pid (
#ifdef UseFunctionPrototypes
        pid_t pid, dir_pane_info *dpi
#endif
);
extern char *GetCWD (
#ifdef UseFunctionPrototypes
	char *path, int size
#endif
);

/*************************************************************************
 * Variables exported
 ************************************************************************/ 

/*************************************************************************
 * Extern functions
 ************************************************************************/ 
extern void builtin_command (
#ifdef UseFunctionPrototypes
	char *command, dir_pane_info *dpi
#endif
);
extern void BusyCursor (
#ifdef UseFunctionPrototypes
	Widget toplevel
#endif
);
extern void UnBusyCursor (
#ifdef UseFunctionPrototypes
	Widget toplevel
#endif
);
extern void PromptDialog (
#ifdef UseFunctionPrototypes
	Widget parent, char *prompt, int modal, char **answer
#endif
);
extern void ConfirmDialog (
#ifdef UseFunctionPrototypes
	Widget parent, char *prompt, int modal, Bool *answer
#endif
);
extern Bool XtChdir (
#ifdef UseFunctionPrototypes
	char *dir, Widget w
#endif
);
extern void XSystem (
#ifdef UseFunctionPrototypes
	char *command, Widget w
#endif
);

/*************************************************************************
 * Extern variables
 ************************************************************************/ 
extern file_action *actions;
extern XmString empty_string;

/*************************************************************************
 * Forward functions
 ************************************************************************/ 
void expand_command (
#ifdef UseFunctionPrototypes
	char **command, char **argv, int argc, char *action
#endif
);
void do_command (
#ifdef UseFunctionPrototypes
	char *command, int run_in_xterm, int pause_after_exec, dir_pane_info *dpi
#endif
);
void expand_prompt ();

/*************************************************************************
 * Local variables
 ************************************************************************/ 

#define XTERM_STR "xterm "
#define DISPLAY_STR " -display "
#define TITLE_STR " -title '"
#define PAUSEME_STR "' -exec pauseme '"
#define EXEC_STR "' -e sh -c '"

#define COMM_SIZE		4096
#define IS_WHITE_SPACE(c)	((c) == ' ' || (c) == '\t' || (c) == '\n')
#define STRING_LEN_CHECK(size)	if ((int)(new_str + (size) - *command) > COMM_SIZE)\
				{	fprintf (stderr, "action too long\n");\
					*new_str = '\0';\
					return;}

#define IS_BUILTIN_COMMAND(s)	(strncmp (s, CHDIR_COMMAND, LEN_CHDIR_COMMAND) == 0 || \
				 strncmp (s, OPENDIR_COMMAND, LEN_OPENDIR_COMMAND) == 0 || \
				 strncmp (s, INFO_COMMAND, LEN_INFO_COMMAND) == 0)

/*************************************************************************
 * Function: expand_command ()
 * Purpose: this expands out the action into a command
 * 	    it uses the ksh patterns of
 *		\c = c
 *		$0 = selected file
 *		$d = n-th parameter in prompt
 *		$* = string of entire prompt 
 *		${d#p} = d-th param less pattern p from front
 *		${d%p} = d-th param less pattern p from end
 * In parameters: argv, argc, action
 * Out parameters: command
 * Precondition: action is a string that can be applied to the file
 * Postcondition: command is the action with patterns replaced
 *                according to params in argv
 ************************************************************************/ 
void
expand_command 
#ifdef UseFunctionPrototypes
	(char **command, char **argv, int argc, char *action)
#else
	(command, argv, argc, action)
	char **command;
	char **argv;
	int argc;
	char *action;

#endif
{
	char *p = NULL;
	char *new_str = NULL;
	int digit;
	char *end_brace_pos = NULL;
	int arg_len, pat_len;

#ifdef DEBUG
	fprintf (stderr, "expanding action: %s\n", action);
#endif
	/* create the new string, hopefully too big */
	*command = new_str = (char *) XtMalloc (4096);

	/* and populate it */
	p = action; 
	while (*p != '\0')
	{	if (*p == '\\')
		{	/* escape sequence */
			STRING_LEN_CHECK (2);
			p++;
			*new_str++ = *p++;
			continue;
		}
		if (*p == '$')
		{	 p++;
			if (isdigit (*p))
			{	/* $d param subst */
				digit = *p++ - '0';
				if (digit >= argc)
					/* off the end of the argv array */
					continue;
				arg_len = strlen (argv[digit]);
				STRING_LEN_CHECK (arg_len);
				strcpy (new_str, argv[digit]);
				new_str += arg_len;
				continue;
			}
			if (*p == '*')
			{	/* $* all params subst */
				int i;

				for (i = 1; i < argc; i++)
				{	arg_len = strlen (argv[i]);
					STRING_LEN_CHECK (arg_len);
					strcpy (new_str, argv[i]);
					new_str += arg_len;
					if (i < argc - 1)
						/* change \0 to ' ' */
						*new_str++ = ' ';
				}
				p++;
				continue;
			}
			if (*p != '{')
			{	/* pass $ through - not one of our patterns */
				STRING_LEN_CHECK (2);
				*new_str++ = '$';
				*new_str++ = *p++;
				continue;
			}
			/* it's '${' so should end in '}' */
			end_brace_pos = strchr (p, '}');
			if (end_brace_pos == NULL)
			{	/* dunno what to do - no closing brace */
				fprintf (stderr, "no closing } in action!\n");
				continue;
			}
			p++;
			if (!isdigit (*p))
			{	/* pass it through - not a positional param */
				STRING_LEN_CHECK ((int)(end_brace_pos - p) + 2);
				*new_str++ = '$';
				*new_str++ = '{';
				while (p != end_brace_pos + 1)
					*new_str++ = *p++;
				continue;
			}
			/* its ${d...}. Is ... NULL, %, #? */
			digit = *p - '0';
			STRING_LEN_CHECK (1);
			p++;
			if (p == end_brace_pos)
			{	/* ${d} param subst */
				arg_len = strlen (argv[digit]);
				STRING_LEN_CHECK (arg_len);
				strcpy (new_str, argv[digit]);	 
				new_str += arg_len;
				p++;	/* skip past '}' */
				continue;
			}
			if (*p == '#')
			{	/* ${d#pat} take pattern off the front */
				p++;
				pat_len = (int) (end_brace_pos - p);
				if (strncmp (argv[digit],p,
					pat_len) != 0)
				{	 /* no match, copy whole arg */
					strcpy (new_str, argv[digit]);
					new_str += strlen (argv[digit]);
					p += pat_len + 1;
					continue;
				}
				/* copy part after match */
				strcpy (new_str, argv[digit] + pat_len);
				new_str +=strlen (argv[digit]) - pat_len;
				p += pat_len + 1;
				continue;
			}
			if (*p == '%')
			{	/* ${d%pat} take pattern off the end */
				int stem_len, arg_len;
				char *tail;

				p++;
				pat_len = (int) (end_brace_pos - p);
				arg_len = strlen (argv[digit]);
				stem_len = arg_len - pat_len;

				/* pattern should start at tail */
				tail = argv[digit] + stem_len;
				if (strncmp (tail, p, pat_len) != 0)
				{	/* no match, copy all */
					strcpy (new_str, argv[digit]);
					new_str += arg_len;
					p += pat_len + 1;
					continue;
				}
				/* copy first part only, upto pattern */
				strncpy (new_str, argv[digit], stem_len);
				new_str += stem_len;
				p += pat_len + 1;
				continue;
			}
			/* dunno what it is */
			fprintf (stderr, "unknown pattern %s in action\n",
					p);
			continue;
		}
		/* copy any other char */
		*new_str++ = *p++;
	}
	*new_str = '\0';
}

/*************************************************************************
 * Function: do_comand ()
 * Purpose: execute the selected command
 * In parameters: command, run_in_xterm, pause_after_exec
 * Out parameters:
 * Precondition: command is a valid Unix command
 * Postcondition: action specified by command running asynchronously
 ************************************************************************/ 
void 
do_command 
#ifdef UseFunctionPrototypes
	(char *command, int run_in_xterm, int pause_after_exec, dir_pane_info *dpi)
#else
	(command, run_in_xterm, pause_after_exec, dpi)
	char *command;
	Bool run_in_xterm;
	Bool pause_after_exec;
        dir_pane_info *dpi;

#endif
{
	int len_new_command;
	char *new_command = NULL;
	char *display_str = NULL;
        int pid;
	static char *shell = NULL;
	static char *shell_basename = NULL;

#ifdef DEBUG
	fprintf (stderr, "original command %s, run in xterm %d, pause_after_exec %d\n",
		command, run_in_xterm, pause_after_exec);
#endif
	/* find out what the user's shell is */
	if (shell == NULL)
		if ((shell = getenv ("SHELL")) == NULL)
			shell = "/bin/sh";
	if (shell_basename == NULL)
	{
		shell_basename = shell + strlen (shell);
		while (shell_basename > shell && *shell_basename != '/')
			shell_basename--;
	}

	/* build up command depending upon environment it runs in */
	if (run_in_xterm && pause_after_exec)
	{
		display_str = DisplayString (XtDisplay (dpi -> toplevel));
		len_new_command = 2 * strlen (command) + 
					strlen (XTERM_STR) +
					strlen (DISPLAY_STR) +
					strlen (display_str) +
					strlen (TITLE_STR) +
					strlen (PAUSEME_STR) + 4;
		/* leave space for \0 terminator */
		new_command = XtMalloc (len_new_command + 1);
		strcpy (new_command, XTERM_STR);
		strcat (new_command, DISPLAY_STR);
		strcat (new_command, display_str);
		strcat (new_command, TITLE_STR);
		strcat (new_command, command);
		strcat (new_command, PAUSEME_STR);
		strcat (new_command, command);
                strcat (new_command, "' ");
        }
	else if (run_in_xterm)
	{
		display_str = DisplayString (XtDisplay (dpi -> toplevel));
		len_new_command = 2 * strlen (command) + 
					strlen (XTERM_STR) + 
					strlen(DISPLAY_STR) +
					strlen (display_str) +
					strlen(TITLE_STR) +
					strlen (EXEC_STR) + 4;
		new_command = XtMalloc (len_new_command + 1);
		strcpy (new_command, XTERM_STR);
		strcat (new_command, DISPLAY_STR);
		strcat (new_command, display_str);
		strcat (new_command, TITLE_STR);
		strcat (new_command, command);
		strcat (new_command, EXEC_STR);
		strcat (new_command, command);
                strcat (new_command, "' ");

	}
	else	/* run as an X application, or into existing xterm */
	{
		len_new_command =  strlen (command) + 13;
		new_command = XtMalloc (len_new_command + 1);
		strcpy (new_command, command);
		/* make sure that if it attempts to read from stdin,
		   it fails (eg. an interactive shell not run in an xterm) */
		strcat (new_command, " < /dev/null ");
	}

#ifdef DEBUG
	fprintf (stderr, "Command to be executed: %s\n", new_command);
#endif

        if ((pid = fork ()) > 0)
        {        /* parent */
                 add_pid (pid, dpi);
        }
        else     /* child */
        {	int n;
	
                for (n = 0; n < sysconf (_SC_OPEN_MAX); n++)
			close (n);
		execl (shell, shell_basename, "-c", new_command, (char *) 0);
		fprintf (stderr, "execl %s failed\n", new_command);
        }                
}
