/*************************************************************************
 * File: $Source: /usr/usrs/xsource/xmfm/RCS/filecb.c,v $
 * Author: Jan Newmarch
 * Last modified: $Date: 1993/04/01 04:15:28 $
 * Version: $Revision: 1.10 $
 * Purpose: This file handles the file operations as callbacks from the
 *	File menu buttons
 *
 * Revision history:
 *	5 Aug 92	fixed failure to clear Toolbar on copy, move, new
 *      25 Aug 92       removed : from dialog names for resource manager
 *	16 Oct 92	caddr_t changed to XtPointer
 *       3 Nov 92       lint-ed
 *	 5 Dec 92	existing file under move, new copy file has a 
 *			dialog to check it
 *       8 Dec 92       closed some memory leaks in XmString usage
 *	21 Dec 92	compiled under g++
 *	25 Mar 93	all pointer vbls grounded on definition, many
 *			XtMallocs changed to XtCalloc to trap pointer bugs
 ************************************************************************/ 

#include "copyright.h"
 
/*************************************************************************
 * System includes
 ************************************************************************/ 
#include <stdio.h> 
#include <sys/param.h>
#include <signal.h>
#include <stdlib.h>

#include <Xm/CascadeB.h>     
#include <Xm/FileSB.h>      
#include <Xm/Label.h>
#include <Xm/PushB.h>    
#include <Xm/RowColumn.h>    
#include <Xm/SelectioB.h>    
#include <Xm/ScrolledW.h>
#include <Xm/ScrollBar.h>

/*************************************************************************
 * Local includes
 ************************************************************************/ 
#include "filecb.h"
#include "const.h"
#include "xmvararg.h"
#include "types.h"
#include "debug_malloc.h"
#include "PaneB.h"
#include "Panel.h"
#include "Toolbar.h"

/*************************************************************************
 * Functions exported
 ************************************************************************/ 
extern void FileNewFileCB (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer client_data, XtPointer call_data
#endif
);
extern void FileNewDirCB (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer client_data, XtPointer call_data
#endif
);
extern void FileCopyCB (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer client_data, XtPointer call_data
#endif
);
extern void FileMoveCB (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer client_data, XtPointer call_data
#endif
);
extern void FileCloseCB (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer client_data, XtPointer call_data
#endif
);
extern void FileExitCB (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer client_data, XtPointer call_data
#endif
);

extern void CanOverwriteExistingFileDialog (
#ifdef UseFunctionPrototypes
	Widget w, char *file, int modal, int *answer
#endif
);

/*************************************************************************
 * Variables exported
 ************************************************************************/ 

/*************************************************************************
 * Extern variables
 ************************************************************************/ 

/*************************************************************************
 * Extern functions
 ************************************************************************/ 
extern void WarningDialog (
#ifdef UseFunctionPrototypes
	Widget parent, char *prompt, int modal, int *answer
#endif
);
extern void DestroyDialogCB (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer c1, XtPointer c2
#endif
);
extern DirectoryMgr *GetFilesInDir (
#ifdef UseFunctionPrototypes
	void
#endif
);
extern void invert_colours (
#ifdef UseFunctionPrototypes
	Widget w
#endif
);
extern void XSystem (
#ifdef UseFunctionPrototypes
        char *command, Widget w
#endif
);
extern Bool XtChdir (
#ifdef UseFunctionPrototypes
        char *dir, Widget w
#endif
);

/*************************************************************************
 * Forward functions
 ************************************************************************/ 

/*************************************************************************
 * Local variables
 ************************************************************************/ 
int pane_count = 1;	/* mumber of window panes open */

/*************************************************************************
 * Function: FileExitCB ()
 * Purpose: terminate the entire application
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition: Exit button pressed in file menu
 * Postcondition:
 ************************************************************************/ 

/* ARGSUSED */
void FileExitCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
Widget		w;		/*  widget id		*/ 
XtPointer		client_data;	/*  data from applicaiton   */ 
XtPointer		call_data;	/*  data from widget class  */ 

#endif
{ 
	int answer;

	/*	Terminate the application. 
	*/ 
	WarningDialog(w, "This will exit xmfm", True, &answer);
	if (answer == OK_ANSWER)
		exit (0);
} 
 

/*************************************************************************
 * Function: NewFileOkCB ()
 * Purpose: create a new file in curent directory
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition: Ok Button pressed in New File dialog
 * Postcondition: new file created, or user informed of error
 ************************************************************************/

/* ARGSUSED */ 
static void 
NewFileOkCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
	Widget		w;		/*  widget id		*/ 
	XtPointer		client_data;	/*  data from applicaiton   */ 
	XtPointer		call_data;

#endif
{	XmString xmstr = NULL;
	char *file_name = NULL;
	char command[MAXPATHLEN + 16];
	dir_pane_info *dpi = NULL;
	int answer;

	XtVaGetValues (w, 
			XmNtextString, (XtArgVal) &xmstr,
			XmNuserData, (XtArgVal) &dpi,
			NULL);
	XtChdir (DirectoryPath (DirectoryMgrDir (dpi -> directory_manager)),
			dpi -> toplevel);

	XmStringGetLtoR (xmstr, XmSTRING_DEFAULT_CHARSET, &file_name);
        XmStringFree (xmstr);

	CanOverwriteExistingFileDialog (w, file_name, True, &answer);
	if (answer != OK_ANSWER)
        {       XtFree (file_name);
		return;
        }

	strcpy (command, "sh -c echo >");
	strcat (command, file_name);
	XSystem (command, w);

	XtUnmanageChild (w);
	XtDestroyWidget (XtParent (w));

	ResetPanesAndSelectFile (dpi, file_name);
        XtFree (file_name);
}


/*************************************************************************
 * Function: NewDirOkCB ()
 * Purpose: create a new dir in current directory
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition: OK button pressed in New Dir dialog
 * Postcondition: new dir created, or user informed of error
 ************************************************************************/ 

/* ARGSUSED */
static void 
NewDirOkCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
	Widget		w;		/*  widget id		*/ 
	XtPointer		client_data;	/*  data from applicaiton   */ 
	XtPointer		call_data;

#endif
{	XmString xmstr = NULL;
	char *dir_name = NULL;
	char command[MAXPATHLEN + 8];
	dir_pane_info *dpi = NULL;

	XtVaGetValues (w, 
			XmNtextString, &xmstr,
			XmNuserData, (XtArgVal) &dpi,
			NULL);
	XtChdir (DirectoryPath (DirectoryMgrDir (dpi -> directory_manager)),
			dpi -> toplevel);

	XmStringGetLtoR (xmstr, XmSTRING_DEFAULT_CHARSET, &dir_name);
        XmStringFree (xmstr);

	strcpy (command, "mkdir ");
	strcat (command, dir_name);
	XSystem (command, w);

	XtUnmanageChild (w);
	XtDestroyWidget (XtParent (w));

	/* now reset panes and select this directory ... */
	ResetPanesAndSelectFile (dpi, dir_name);
        XtFree (dir_name);
}


/*************************************************************************
 * Function: MoveFileOkCB ()
 * Purpose: 
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition: OK button pressed in Move File dialog
 * Postcondition: file moved to new file, which is got from a prompt
 ************************************************************************/

/* ARGSUSED */ 
static void 
MoveFileOkCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
	Widget		w;		/*  widget id		*/ 
	XtPointer		client_data;	/*  data from applicaiton   */ 
	XtPointer		call_data;

#endif
{	XmString xmstr = NULL;
	char *new_file = NULL;
	char command[MAXPATHLEN + 6];
	dir_pane_info *dpi = NULL;
	int answer;

	XtVaGetValues (w, 
			XmNtextString, &xmstr,
			XmNuserData, (XtArgVal) &dpi,
			NULL);
	XtChdir (DirectoryPath (DirectoryMgrDir (dpi -> directory_manager)),
			dpi -> toplevel);

	XmStringGetLtoR (xmstr, XmSTRING_DEFAULT_CHARSET, &new_file);
        XmStringFree (xmstr);

	CanOverwriteExistingFileDialog (w, new_file, True, &answer);
	if (answer != OK_ANSWER)
        {       XtFree (new_file);
		return;
        }

	strcpy (command, "mv ");
	strcat (command, dpi -> file_selected);
	strcat (command, " ");
	strcat (command, new_file);
	strcat (command, " < /dev/null");
	XSystem (command, w);

	XtUnmanageChild (w);
	XtDestroyWidget (XtParent (w));

	/* now reset panes and select this file ... */
	ResetPanesAndSelectFile (dpi, new_file);
        XtFree (new_file);
}

/*************************************************************************
 * Function: CopyFileOkCB
 * Purpose: 
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition: OK button pressed in Copy File dialog
 * Postcondition: file copied to new file, which is got from a prompt
 ************************************************************************/ 

/* ARGSUSED */
static void 
CopyFileOkCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
	Widget		w;		/*  widget id		*/ 
	XtPointer		client_data;	/*  data from applicaiton   */ 
	XtPointer		call_data;

#endif
{	XmString xmstr = NULL;
	char *new_file = NULL;
	char command[MAXPATHLEN + 8];
	dir_pane_info *dpi = NULL;
	int answer;

	XtVaGetValues (w, 
			XmNtextString, (XtArgVal) &xmstr,
			XmNuserData, (XtArgVal) &dpi,
			NULL);
	XtChdir (DirectoryPath (DirectoryMgrDir (dpi -> directory_manager)),
			dpi -> toplevel);

	XmStringGetLtoR (xmstr, XmSTRING_DEFAULT_CHARSET, &new_file);
        XmStringFree (xmstr);

	CanOverwriteExistingFileDialog (w, new_file, True, &answer);
	if (answer != OK_ANSWER)
        {       XtFree (new_file);
		return;
        }

	/* -r option allows us to copy directories as well as files: */
	strcpy (command, "cp -r ");
	
	strcat (command, dpi -> file_selected);
	strcat (command, " ");
	strcat (command, new_file);
	strcat (command, " < /dev/null");
	XSystem (command, w);

	/* now reset panes and select this file ... */
	ResetPanesAndSelectFile (dpi, new_file);
        XtFree (new_file);

	XtUnmanageChild (w);
	XtDestroyWidget (XtParent (w));
}

/*************************************************************************
 * Function: FileNewFileCB ()
 * Purpose: create a dialog to ask for new file name
 *	with callbacks to handle OK (create file), Cancel
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition: New file button pressed in File menu
 * Postcondition: New File.. dialog showing
 ************************************************************************/

/* ARGSUSED */ 
void
FileNewFileCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
	Widget		w;		/*  widget id		*/ 
	XtPointer		client_data;	/*  data from applicaiton   */ 
	XtPointer		call_data;

#endif
{	Widget new_file = NULL;
	XtArgVal user_data;
	Widget help_button = NULL;

	XtVaGetValues (w, XmNuserData, &user_data, NULL);
	new_file = XmVaCreatePromptDialog (w, "new file name",
				XmNuserData, user_data,
				NULL);
	XtAddCallback (new_file, XmNokCallback, NewFileOkCB, NULL);
	XtAddCallback (new_file, XmNcancelCallback, DestroyDialogCB, NULL);

	help_button = XmSelectionBoxGetChild (new_file,
				XmDIALOG_HELP_BUTTON);
	XtUnmanageChild (help_button);

	XtManageChild(new_file);
}



/*************************************************************************
 * Function: FileNewDirCB ()
 * Purpose: create a dialog to ask for a new directory name
 *	with callbacks to handle OK (create dir), Cancel
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition: New Dir button pressed in File menu
 * Postcondition: dialog showing
 ************************************************************************/ 

/* ARGSUSED */
void
FileNewDirCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
	Widget		w;		/*  widget id		*/ 
	XtPointer		client_data;	/*  data from applicaiton   */ 
	XtPointer		call_data;

#endif
{	Widget new_dir = NULL;
	Widget help_button = NULL;
	XtArgVal user_data;

	XtVaGetValues (w, XmNuserData, &user_data, NULL);
	new_dir = XmVaCreatePromptDialog (w, "new dir name",
			XmNuserData, user_data,
			NULL);
	XtAddCallback (new_dir, XmNokCallback, NewDirOkCB, NULL);
	XtAddCallback (new_dir, XmNcancelCallback, DestroyDialogCB, NULL);

	help_button = XmSelectionBoxGetChild (new_dir,
				XmDIALOG_HELP_BUTTON);
        XtUnmanageChild (help_button);

	XtManageChild(new_dir);
}




/*************************************************************************
 * Function: FileMoveCB ()
 * Purpose: move a file to new prompted name
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition: Move button pressed in file menu
 * Postcondition: dialog showing asking for new file name
 ************************************************************************/ 

/* ARGSUSED */
void
FileMoveCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
	Widget		w;		/*  widget id		*/ 
	XtPointer		client_data;	/*  data from applicaiton   */ 
	XtPointer		call_data;

#endif
{	Widget new_file = NULL;
	dir_pane_info *dpi = NULL;
	int dummy;

	XtVaGetValues (w, XmNuserData, &dpi, NULL);
	if (dpi -> file_selected == NULL)
	{
		WarningDialog (w, "Must select a file to move",
				True, &dummy);
		return;
	}

	new_file = XmVaCreatePromptDialog (w, "move file",
				XmNuserData, dpi, NULL);
	XtAddCallback (new_file, XmNokCallback, MoveFileOkCB, NULL);
	XtAddCallback (new_file, XmNcancelCallback, DestroyDialogCB, NULL);
	XtManageChild(new_file);
}


/*************************************************************************
 * Function: FileCopyCB ()
 * Purpose: copy a file to new name
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition: Copy button pressed in File menu
 * Postcondition: dialog showing asking for new file name
 ************************************************************************/ 

/* ARGSUSED */
void
FileCopyCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
	Widget		w;		/*  widget id		*/ 
	XtPointer		client_data;	/*  data from applicaiton   */ 
	XtPointer		call_data;

#endif
{	Widget new_file = NULL;
	dir_pane_info *dpi = NULL;
	int dummy;

	XtVaGetValues (w, XmNuserData, &dpi, NULL);
	if (dpi -> file_selected == NULL)
	{
		WarningDialog (w, "Must select a file to copy",
				True, &dummy);
		return;
	}

	new_file = XmVaCreatePromptDialog (w, "copy file",
				XmNuserData, dpi, NULL);
	XtAddCallback (new_file, XmNokCallback, CopyFileOkCB, NULL);
	XtAddCallback (new_file, XmNcancelCallback, DestroyDialogCB, NULL);
	XtManageChild(new_file);
}


/*************************************************************************
 * Function: FileCloseCB ()
 * Purpose: close a pane on a directory
 *	if last pane, application exits after user confirmation
 * In parameters: w, client_data, call_data
 * Out parameters: 
 * Precondition: Close button pressed in File menu
 * Postcondition: current pane destroyed
 ************************************************************************/ 

/* ARGSUSED */
void
FileCloseCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
	Widget		w;		/*  widget id		*/ 
	XtPointer		client_data;	/*  data from applicaiton   */ 
	XtPointer		call_data;

#endif
{	int answer;
	dir_pane_info *dpi = NULL;

	if (pane_count == 1)
	{
		WarningDialog(w, "This will exit xmfm", True, &answer);
		if (answer == OK_ANSWER)
			exit (0);
		return;
	}

	/* reclaim memory from components */
	XtVaGetValues (w, XmNuserData, &dpi, NULL);
	if (dpi -> reference_count > 0)
		/* a child is still alive, expecting to use this */
		dpi -> being_destroyed = True;
	else
	{
		DirectoryMgrClose (dpi -> directory_manager);
		XtFree ((char *) dpi);
	}
	
	/* hop up to dropdown shell */
	while ( !XtIsShell (w))
		w = XtParent (w);
	/* and to the main window */
	w = XtParent (w);
	/* hop up to toplevel shell */
	while ( !XtIsShell (w))
		w = XtParent (w);
	XtDestroyWidget (w);

	/* memory leaks in here - should reclaim
	   dpi stuff, DirMgr, etc
	*/
	
	pane_count--;
}


