/*************************************************************************
 * File: $Source: /usr/usrs/xsource/xmfm/RCS/filtercb.c,v $
 * Author: Jan Newmarch
 * Last modified: $Date: 1993/04/01 04:15:30 $
 * Version: $Revision: 1.6 $
 * Purpose: This file handles the directory goto requests from the Goto
 *	menu, through callbacks attached to each button
 *
 * Revision history:
 *	16 Oct 92	caddr_t changed to XtPointer
 *       3 Nov 92       lint-ed
 *	21 Nov 92	fixed crash on cancelling filter dialog
 *	25 Mar 93	all pointer vbls grounded on definition, many
 *			XtMallocs changed to XtCalloc to trap pointer bugs
 ************************************************************************/ 

#include "copyright.h"

/*************************************************************************
 * System includes
 ************************************************************************/ 
#include <stdlib.h>
#include <stdio.h> 

#include <Xm/CascadeB.h>     
#include <Xm/FileSB.h>      
#include <Xm/Label.h>
#include <Xm/PushB.h>    
#include <Xm/RowColumn.h>    
#include <Xm/SelectioB.h>    

/*************************************************************************
 * Local includes
 ************************************************************************/ 
#include "filecb.h"
#include "types.h"
#include "RegExp.h"
#include "Panel.h"
#include "PaneB.h"
#include "Toolbar.h"
#include "debug_malloc.h"

/*************************************************************************
 * Functions exported
 ************************************************************************/ 
extern void FilterFileCB (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer client_data, XtPointer call_data
#endif
);
extern void FilterDirCB (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer client_data, XtPointer call_data
#endif
);
extern void FilterExecutableCB (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer client_data, XtPointer call_data
#endif
);

/*************************************************************************
 * Variables exported
 ************************************************************************/ 

/*************************************************************************
 * Extern variables
 ************************************************************************/ 

/*************************************************************************
 * Extern functions
 ************************************************************************/ 
extern void GotoCurrentDir (
#ifdef UseFunctionPrototypes
	Widget w
#endif
);
extern void PromptDialog (
#ifdef UseFunctionPrototypes
	Widget parent, char *prompt, int modal, char **answer
#endif
);
extern void PaneButtonInvertColours (
#ifdef UseFunctionPrototypes
	Widget w
#endif
);

/*************************************************************************
 * Forward functions
 ************************************************************************/ 

/*************************************************************************
 * Local variables
 ************************************************************************/ 

/*************************************************************************
 * Function: FilterFileCB ()
 * Purpose: set the filter for ordinary files
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition;
 * Postcondition:
 ************************************************************************/ 

/* ARGSUSED */
void
FilterFileCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
Widget		w;		/*  widget id		*/ 
XtPointer		client_data;	/*  data from applicaiton   */ 
XtPointer		call_data;	/*  data from widget class  */ 

#endif
{	dir_pane_info *dpi = NULL;
        char regexp[FSM_LENGTH]; 
	char *filter = NULL;

	XtVaGetValues (w, XmNuserData, &dpi, NULL);

/*
	XtFree (dpi -> file_filter);
	XtFree (dpi -> file_filter_regexp);
*/

	PromptDialog (w, "File filter", True, &filter);
	if (filter == NULL)
		return;		/* dialog cancelled */
	dpi -> file_filter = filter;

	RegExpPatternToRegExp (dpi -> file_filter, regexp);
	RegExpCompile (regexp, dpi -> file_filter_regexp, FSM_LENGTH);

	/* empty out current selection, in case it is no longer visible */
	if (dpi -> file_selected != NULL)
	{	PaneButtonInvertColours (dpi -> file_button_selected);
		dpi -> file_selected = NULL;
		ToolbarClear (dpi -> files_toolbar);
	}
        PanelSetFiltersInLabels (dpi);
        PanelSetFiles (dpi);
}
 
/*************************************************************************
 * Function: FilterDirCB ()
 * Purpose: set the filter on ditectory names
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition;
 * Postcondition: 
 ************************************************************************/ 

/* ARGSUSED */
void
FilterDirCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
Widget		w;		/*  widget id		*/ 
XtPointer		client_data;	/*  data from applicaiton   */ 
XtPointer		call_data;

#endif
{	dir_pane_info *dpi = NULL;
        char regexp[FSM_LENGTH]; 
	char *filter = NULL;

	XtVaGetValues (w, XmNuserData, &dpi, NULL);

/*
	XtFree (dpi -> dir_filter);
	XtFree (dpi -> dir_filter_regexp);
*/

	PromptDialog (w, "Directory filter", True, &filter);
	if (filter == NULL)
		return;		/* dialog cancelled */
	dpi -> dir_filter = filter;

	RegExpPatternToRegExp (dpi -> dir_filter, regexp);
	RegExpCompile (regexp, dpi -> dir_filter_regexp, FSM_LENGTH);

	/* empty out current selection, in case it is no longer visible */
	if (dpi -> file_selected != NULL)
	{	PaneButtonInvertColours (dpi -> file_button_selected);
		dpi -> file_selected = NULL;
		ToolbarClear (dpi -> files_toolbar);
	}

        PanelSetFiltersInLabels (dpi);

        PanelSetFiles (dpi);
}

/*************************************************************************
 * Function: FilterExecutbaleCb ()
 * Purpose: set file filter on executbale files
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition; 
 * Postcondition: 
 ************************************************************************/ 

/* ARGSUSED */
void
FilterExecutableCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
Widget		w;		/*  widget id		*/ 
XtPointer		client_data;	/*  data from applicaiton   */ 
XtPointer		call_data;	/*  data from widget class  */ 

#endif
{	dir_pane_info *dpi = NULL;
        char regexp[FSM_LENGTH]; 
	char *filter = NULL;

	XtVaGetValues (w, XmNuserData, &dpi, NULL);

	/* memory leak? clean up old data? 
	XtFree (dpi -> executable_filter);
	XtFree (dpi -> executable_filter_regexp);
        */

	PromptDialog (w, "Executable filter", True,
			&filter);
	if (filter == NULL)
		return;		/* dialog cancelled */
	dpi -> executable_filter = filter;

	RegExpPatternToRegExp (dpi -> executable_filter, regexp);
	RegExpCompile (regexp, dpi -> executable_filter_regexp, FSM_LENGTH);

	/* empty out current selection, in case it is no longer visible */
	if (dpi -> file_selected != NULL)
	{	PaneButtonInvertColours (dpi -> file_button_selected);
		dpi -> file_selected = NULL;
		ToolbarClear (dpi -> files_toolbar);
	}

        PanelSetFiltersInLabels (dpi);

        PanelSetFiles (dpi);
}
