/*************************************************************************
 * File: $Source: /usr/usrs/xsource/xmfm/RCS/info.c,v $
 * Author: Jan Newmarch
 * Last modified: $Date: 1993/04/01 04:15:35 $
 * Version: $Revision: 1.6 $
 * Purpose:
 *
 * Revision history:
 *      22 Aug 92      made info dialog a per pane object, accessible via dpi
 *	8  Sep 92      fixed bug that used run dialog instead of info dialog
 *       3 Nov 92      lint-ed
 *	11 Nov 92      follow symbolic links to actual info, ignore link file
 *	12 Nov 92      file permissions can now be edited
 *	 2 Dec 92      added group info to info dialog
 *	 3 Dec 92      added modified time, link count to dialog
 *       8 Dec 92      fixed dangling pointer in dpi -> file_selected on
 *                     directory refresh
 *	25 Mar 93	all pointer vbls grounded on definition, many
 *			XtMallocs changed to XtCalloc to trap pointer bugs
 ************************************************************************/ 

#include "copyright.h"

/*************************************************************************
 * System includes
 ************************************************************************/ 
#include <pwd.h>
#include <string.h>
#include <Xm/SelectioB.h>
#include <Xm/Label.h>
#include <Xm/ToggleB.h>
#include <Xm/RowColumn.h>
#include <Xm/Form.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <grp.h>
#include <time.h>

/*************************************************************************
 * Local includes
 ************************************************************************/ 
#include "const.h"
#include "types.h"
#include "xmvararg.h"
#include "DirMgr.h"
#include "debug_malloc.h"

/*************************************************************************
 * Functions exported
 ************************************************************************/ 
void builtin_info (
#ifdef UseFunctionPrototypes
	dir_pane_info *dpi
#endif
);

/*************************************************************************
 * Variables exported
 ************************************************************************/ 

/*************************************************************************
 * Extern variables
 ************************************************************************/ 

/*************************************************************************
 * Extern functions
 ************************************************************************/ 
extern void WarningDialog (
#ifdef UseFunctionPrototypes
	Widget parent, char *prompt, int modal, int *answer
#endif
);
extern void ResetPanesAndSelectFile (
#ifdef UseFunctionPrototypes
        dir_pane_info *dpi, char *orig_file_name
#endif
);

/*************************************************************************
 * Forward functions
 ************************************************************************/ 

/*************************************************************************
 * Local variables
 ************************************************************************/ 
#define EXECUTABLE(m)	((m) & (S_IXUSR | S_IXGRP | S_IXOTH))
static Widget	dialog = NULL;

/* its a bit gross having these all as static, but we have to keep them
   around somewhere if we are going to reuse the dialog instead of 
   recreating it each time
*/
static	Widget	form = NULL,
                filename_label = NULL, 
		filename = NULL,
                separator1 = NULL,
                separator2 = NULL,
                separator3 = NULL,
                separator4 = NULL,
                separator5 = NULL,
		size_label = NULL,
		size = NULL,
		owner_label = NULL,
		owner = NULL,
		group_str = NULL,
		group_str_label = NULL,
		description_label = NULL,
		description = NULL,
		link_count_label = NULL,
		link_count = NULL,
		last_modified_label = NULL,
		last_modified = NULL,
		perms_label = NULL,
		perms_rowcol = NULL,
		empty_label = NULL,
		read_label = NULL,
		write_label = NULL,
                exec_label = NULL,
		user_label = NULL,
		group_label = NULL,
		other_label = NULL,
		ur_toggle = NULL,
		uw_toggle = NULL,
		ux_toggle = NULL,
		gr_toggle = NULL,
		gw_toggle = NULL,
		gx_toggle = NULL,
		or_toggle = NULL,
		ow_toggle = NULL,
		ox_toggle = NULL;

static Bool mode_changed;	/* has the user changed file mode? */

/*************************************************************************
 * Function	   : OkCB ()
 * Purpose         : change file mode if needed when ok button pressed
 * In parameters   : w, client_data, call_data
 * Out parameters  :
 * Side effects    : 
 * Function returns:
 * Precondition    : 
 * Postcondition   :
 ************************************************************************/ 
static void
OkCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)
	Widget w;
	XtPointer client_data;
	XtPointer call_data;

#endif
{
	dir_pane_info *dpi = NULL;
	DirEntry *de = NULL;
	Boolean button_set;
	Arg args[1];
	char *file_selected = NULL;
#ifdef NO_MODE_T
	int new_mode, old_mode;
#else
	mode_t new_mode, old_mode;
#endif

#ifdef DEBUG
	fprintf (stderr, "mode_changed: %d\n", mode_changed);
#endif
	if ( !mode_changed)
		return;
		
	new_mode = 0;
	/* build the new mode from the state of the mode buttons */
	XtSetArg (args[0], XmNset, &button_set);
	XtGetValues (ur_toggle, args, 1);
	if (button_set)
		new_mode |= S_IRUSR;
	XtGetValues (uw_toggle, args, 1);
	if (button_set)
		new_mode |= S_IWUSR;
	XtGetValues (ux_toggle, args, 1);
	if (button_set)
		new_mode |= S_IXUSR;
	XtGetValues (gr_toggle, args, 1);
	if (button_set)
		new_mode |= S_IRGRP;
	XtGetValues (gw_toggle, args, 1);
	if (button_set)
		new_mode |= S_IWGRP;
	XtGetValues (gx_toggle, args, 1);
	if (button_set)
		new_mode |= S_IXGRP;
	XtGetValues (or_toggle, args, 1);
	if (button_set)
		new_mode |= S_IROTH;
	XtGetValues (ow_toggle, args, 1);
	if (button_set)
		new_mode |= S_IWOTH;
	XtGetValues (ox_toggle, args, 1);
	if (button_set)
		new_mode |= S_IXOTH;

	XtVaGetValues (w,
		XmNuserData, (XtArgVal) &dpi,
		NULL);

	file_selected = dpi -> file_selected;
	DirectoryMgrGotoNamedItem (dpi -> directory_manager,
					file_selected);
	de = DirectoryMgrCurrentEntry (dpi -> directory_manager);

	/* has the user really made it different? */
	if (new_mode == FileInfoProt (DirEntryActualInfo (de)))
		return;		/* no they didn't */

	/* actually change the mode */
	if (chmod (file_selected, new_mode) == -1)
		return; 	/* oh well, couldn't change it */

	/* reset our stored info */
	old_mode = FileInfoProt (DirEntryActualInfo (de));
	FileInfoProt (DirEntryActualInfo (de)) = new_mode;

	/* if it is a dir, nothing left to do.
	   if it is now executable but wasn't before, or vice
	   versa, need to refresh panes
	*/
	if (DirEntryIsDir (de) ||
		DirEntryIsSymLink (de) &&
			S_ISDIR (FileInfoOrigMode (DirEntryActualInfo (de))))
		return;

        /* make a copy of selected file to avoid dangling pointer after
           directory reset */
        file_selected = NULL;
        if (dpi -> file_selected != NULL)
        {     file_selected = XtMalloc (strlen (dpi -> file_selected) + 1);
              strcpy (file_selected, dpi -> file_selected);
        }

	if ((EXECUTABLE (old_mode) && ! EXECUTABLE (new_mode)) ||
	    ( ! EXECUTABLE (old_mode) && EXECUTABLE (new_mode)))
		ResetPanesAndSelectFile (dpi, file_selected);

        if (file_selected != NULL) XtFree (file_selected);
}

/*************************************************************************
 * Function	   : toggleCB ()
 * Purpose         : set mode_changed to true
 * In parameters   : w, client_data, call_data
 * Out parameters  :
 * Side effects    : 
 * Function returns:
 * Precondition    : 
 * Postcondition   :
 ************************************************************************/ 
void
toggleCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)
	Widget w;
	XtPointer client_data;
	XtPointer call_data;

#endif
{
	mode_changed = True;
}

/*************************************************************************
 * Function	   : builtin_info ()
 * Purpose         : display info on a file, such as size, permissions, etc
 * In parameters   : dpi
 * Out parameters  :
 * Side effects    : creates and shows a file display dialog
 * Function returns:
 * Precondition    : 
 * Postcondition   :
 ************************************************************************/ 
void
builtin_info 
#ifdef UseFunctionPrototypes
	(dir_pane_info *dpi)
#else
	(dpi)
	dir_pane_info *dpi;

#endif
{	
	DirEntry	*de = NULL;
	long		long_size;
	char		str_size[1024];
	XmString	xm_size = NULL;
	char		link_number_buf[16];
	short		link_number;
	int		uid;
	struct passwd	*pw_entry = NULL;
        struct group    *grp_entry = NULL;
	XmString	xm_pw_name = NULL;
	XmString	xm_grp_name = NULL;
	XmString	xm_file_selected = NULL;
        XmString        xm_description = NULL;
        XmString        xm_mod_time = NULL;
	XmString	xm_link_number = NULL;
        static XmString	xm_empty = NULL;
	static short	permissions;
	static Bool	state;
	int		dummy;
	time_t		mod_time;
	char		mod_time_buf [2048];
	

	if (dpi -> file_selected == NULL)
	{	WarningDialog (dpi -> toplevel,
			"Can only show information on a selected file",
			True, &dummy);
		return;
	}
        dialog = dpi -> info_dialog;

	if (dialog == NULL)
	{
		xm_empty = XmStringCreateSimple ("");
		/* create a "prompt" dialog with the prompt bits left out */
		dialog = XmVaCreatePromptDialog (dpi -> file_button_selected,
					"information",
					NULL);
                dpi -> info_dialog = dialog;
		XtUnmanageChild ( 
			XmSelectionBoxGetChild (dialog, XmDIALOG_TEXT));
		XtUnmanageChild ( 
			XmSelectionBoxGetChild (dialog, XmDIALOG_SELECTION_LABEL));
		XtUnmanageChild ( 
			XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON));

		XtAddCallback (dialog, XmNokCallback, OkCB, NULL);
	
		/* add the widget we want as the work area */
		form = XmVaCreateForm (dialog, "form",
				NULL);
		
		/* put stuff in the form */
		filename_label = XmVaCreateLabel (form, "Filename",
				NULL);
#ifdef  FILENAME_IS_EDITABLE
		/* uses text string - in case we want to change filename.
		   but it makes geometry harder */
		filename = XmVaCreateText (form, "filename",
                                XmNeditable, (XtArgVal) False,
				NULL);
#else
		filename = XmVaCreateLabel (form, "filename",
				NULL);
#endif /* FILENAME_IS_EDITABLE */
                separator1 = XmVaCreateSeparator (form, "separator1",
                                XmNseparatorType, (XtArgVal) XmNO_LINE,
                                NULL);
                separator2 = XmVaCreateSeparator (form, "separator2",
                                XmNseparatorType, (XtArgVal) XmNO_LINE,
                                NULL);
                separator3 = XmVaCreateSeparator (form, "separator3",
                                XmNseparatorType, (XtArgVal) XmNO_LINE,
                                NULL);
                separator4 = XmVaCreateSeparator (form, "separator4",
                                XmNseparatorType, (XtArgVal) XmNO_LINE,
                                NULL);
                separator5 = XmVaCreateSeparator (form, "separator5",
                                XmNseparatorType, (XtArgVal) XmNO_LINE,
                                NULL);
		size_label = XmVaCreateLabel (form, "Size",
				NULL);
		size = XmVaCreateLabel (form, "size",
				NULL);
		owner_label = XmVaCreateLabel (form, "Owner",
				NULL);
		owner = XmVaCreateLabel (form, "owner",
				NULL);
		group_str_label = XmVaCreateLabel (form, "Group",
				NULL);
		group_str = XmVaCreateLabel (form, "group",
				NULL);
		description_label = XmVaCreateLabel (form, "Description",
				NULL);
		description = XmVaCreateLabel (form, "description",
                                XmNalignment, (XtArgVal) XmALIGNMENT_BEGINNING,
				NULL);
		link_count_label = XmVaCreateLabel (form, "Link count",
				NULL);
		link_count = XmVaCreateLabel (form, "link_count",
				NULL);
		last_modified_label = XmVaCreateLabel (form, "Last modified",
				NULL);
		last_modified = XmVaCreateLabel (form, "last_modified",
				NULL);
		perms_label = XmVaCreateLabel (form, "Permissions",
				NULL);
		perms_rowcol = XmVaCreateForm (form, "perms_rowcol",
/*				XmNpacking,     (XtArgVal) XmPACK_COLUMN,
				XmNnumColumns,  (XtArgVal) 4,
				XmNborderWidth, (XtArgVal) 1,
                                XmNisAligned,   (XtArgVal) False,
*/
				NULL);
		
		/* put stuff in the row col */
		empty_label = XmVaCreateLabel (perms_rowcol, "empty",
				NULL);
		read_label = XmVaCreateLabel (perms_rowcol, "read",
                                XmNalignment, (XtArgVal) XmALIGNMENT_END,
                                XmNtopAttachment,  (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 25,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 0,
                                XmNrightAttachment,(XtArgVal) XmATTACH_POSITION,
                                XmNrightPosition,  (XtArgVal) 25,
				NULL);
		write_label = XmVaCreateLabel (perms_rowcol, "write",
                                XmNalignment, (XtArgVal) XmALIGNMENT_END,
                                XmNalignment, (XtArgVal) XmALIGNMENT_END,
                                XmNtopAttachment,  (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 50,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 0,
                                XmNrightAttachment,(XtArgVal) XmATTACH_POSITION,
                                XmNrightPosition,  (XtArgVal) 25,
				NULL);
		exec_label = XmVaCreateLabel (perms_rowcol, "execute",
                                XmNalignment, (XtArgVal) XmALIGNMENT_END,
                                 XmNtopAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 75,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 0,
                                XmNrightAttachment,(XtArgVal) XmATTACH_POSITION,
                                XmNrightPosition,  (XtArgVal) 25,
				NULL);
	
		user_label = XmVaCreateLabel (perms_rowcol, "user",
                                XmNtopAttachment,  (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 0,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 25,
				NULL);
		ur_toggle = XmVaCreateToggleButton (perms_rowcol, "ur",
                                XmNalignment, (XtArgVal) XmALIGNMENT_END,
                                XmNtopAttachment,  (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 25,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 28,
				NULL);
		uw_toggle = XmVaCreateToggleButton (perms_rowcol, "uw",
                                XmNalignment, (XtArgVal) XmALIGNMENT_END,
                                XmNtopAttachment,  (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 50,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 28,
				NULL);
		ux_toggle = XmVaCreateToggleButton (perms_rowcol, "ux",
                                XmNalignment, (XtArgVal) XmALIGNMENT_END,
                                XmNtopAttachment,  (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 75,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 28,
				NULL);

		group_label = XmVaCreateLabel (perms_rowcol, "group",
                                XmNtopAttachment,  (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 0,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 50,
				NULL);
		gr_toggle = XmVaCreateToggleButton (perms_rowcol, "gr",
                                XmNalignment, (XtArgVal) XmALIGNMENT_END,
                                XmNtopAttachment,  (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 25,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 53,
				NULL);
		gw_toggle = XmVaCreateToggleButton (perms_rowcol, "gw",
                                XmNalignment, (XtArgVal) XmALIGNMENT_END,
                                XmNtopAttachment,  (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 50,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 53,
				NULL);
		gx_toggle = XmVaCreateToggleButton (perms_rowcol, "gx",
                                XmNalignment, (XtArgVal) XmALIGNMENT_END,
                                XmNtopAttachment,  (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 75,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 53,
				NULL);

		other_label = XmVaCreateLabel (perms_rowcol, "other",
                                XmNalignment, (XtArgVal) XmALIGNMENT_BEGINNING,
                                XmNalignment, (XtArgVal) XmALIGNMENT_END,
                                XmNtopAttachment,  (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 0,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 75,
				NULL);
		or_toggle = XmVaCreateToggleButton (perms_rowcol, "or",
                                XmNalignment, (XtArgVal) XmALIGNMENT_END,
                                XmNtopAttachment,  (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 25,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 78,
				NULL);
		ow_toggle = XmVaCreateToggleButton (perms_rowcol, "ow",
                                XmNalignment, (XtArgVal) XmALIGNMENT_END,
                                XmNtopAttachment,  (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 50,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 78,
				NULL);
		ox_toggle = XmVaCreateToggleButton (perms_rowcol, "ox",
                                XmNalignment, (XtArgVal) XmALIGNMENT_END,
                                XmNtopAttachment,  (XtArgVal) XmATTACH_POSITION,
                                XmNtopPosition,    (XtArgVal) 75,
                                XmNleftAttachment, (XtArgVal) XmATTACH_POSITION,
                                XmNleftPosition,   (XtArgVal) 78,
				NULL);

                /* make all these buttons insensitive */
/*
                XtSetSensitive (ur_toggle, False);
                XtSetSensitive (uw_toggle, False);
                XtSetSensitive (ux_toggle, False);
                XtSetSensitive (gr_toggle, False);
                XtSetSensitive (gw_toggle, False);
                XtSetSensitive (gx_toggle, False);
                XtSetSensitive (or_toggle, False);
                XtSetSensitive (ow_toggle, False);
                XtSetSensitive (ox_toggle, False);
*/
		/* callbacks */
		XtAddCallback (ur_toggle, XmNarmCallback, toggleCB, NULL);
		XtAddCallback (uw_toggle, XmNarmCallback, toggleCB, NULL);
		XtAddCallback (ux_toggle, XmNarmCallback, toggleCB, NULL);
		XtAddCallback (gr_toggle, XmNarmCallback, toggleCB, NULL);
		XtAddCallback (gw_toggle, XmNarmCallback, toggleCB, NULL);
		XtAddCallback (gx_toggle, XmNarmCallback, toggleCB, NULL);
		XtAddCallback (or_toggle, XmNarmCallback, toggleCB, NULL);
		XtAddCallback (ow_toggle, XmNarmCallback, toggleCB, NULL);
		XtAddCallback (ox_toggle, XmNarmCallback, toggleCB, NULL);
	
	
		/* manage the lot */
		XtManageChild (filename_label);
		XtManageChild (filename);
                XtManageChild (separator1);
                XtManageChild (separator2);
                XtManageChild (separator3);
                XtManageChild (separator4);
                XtManageChild (separator5);
		XtManageChild (size_label);
		XtManageChild (size);
		XtManageChild (owner_label);
		XtManageChild (owner);
		XtManageChild (group_str_label);
		XtManageChild (group_str);
		XtManageChild (link_count_label);
		XtManageChild (link_count);
		XtManageChild (last_modified_label);
		XtManageChild (last_modified);
		XtManageChild (description_label);
		XtManageChild (description);
		XtManageChild (perms_label);
		XtManageChild (empty_label);
		XtManageChild (read_label);
		XtManageChild (write_label);
                XtManageChild (exec_label);
		XtManageChild (user_label);
		XtManageChild (group_label);
		XtManageChild (other_label);
		XtManageChild (ur_toggle);
		XtManageChild (uw_toggle);
		XtManageChild (ux_toggle);
		XtManageChild (gr_toggle);
		XtManageChild (gw_toggle);
		XtManageChild (gx_toggle);
		XtManageChild (or_toggle);
		XtManageChild (ow_toggle);
		XtManageChild (ox_toggle);
		XtManageChild (perms_rowcol);
		XtManageChild (form);
	
		/* the geometry for this mess */
		XtVaSetValues (filename_label,
			XmNtopAttachment,	(XtArgVal) XmATTACH_FORM,
			XmNrightAttachment,	(XtArgVal) XmATTACH_POSITION,
			XmNrightPosition,	(XtArgVal) 40,
			NULL);
		XtVaSetValues (filename,
			XmNtopAttachment,	(XtArgVal) XmATTACH_FORM,
			XmNleftAttachment,	(XtArgVal) XmATTACH_POSITION,
			XmNleftPosition,        (XtArgVal) 41,
                        NULL);
		XtVaSetValues (separator1,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,	        (XtArgVal) filename,
			XmNleftAttachment,	(XtArgVal) XmATTACH_FORM,
			XmNrightAttachment,	(XtArgVal) XmATTACH_FORM,
			NULL);
		XtVaSetValues (description_label,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) separator1,
			XmNleftAttachment,	(XtArgVal) XmATTACH_FORM,
			NULL);
		XtVaSetValues (description,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) separator1,
			XmNleftAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNleftWidget,		(XtArgVal) description_label,
			XmNrightAttachment,	(XtArgVal) XmATTACH_FORM,
			NULL);
		XtVaSetValues (separator2,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,	        (XtArgVal) description,
			XmNleftAttachment,	(XtArgVal) XmATTACH_FORM,
			XmNrightAttachment,	(XtArgVal) XmATTACH_FORM,
			NULL);
		XtVaSetValues (size_label,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) separator2,
			XmNleftAttachment,	(XtArgVal) XmATTACH_FORM,
			NULL);
		XtVaSetValues (size,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) separator2,
			XmNleftAttachment,	(XtArgVal) XmATTACH_WIDGET,
                        XmNleftWidget,          (XtArgVal) size_label,
 			XmNrightAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNrightWidget,		(XtArgVal) link_count_label,
			NULL);
		XtVaSetValues (link_count_label,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) separator2,
			XmNleftAttachment,	(XtArgVal) XmATTACH_POSITION,
                        XmNleftPosition,        (XtArgVal) 50,
			NULL);
		XtVaSetValues (link_count,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) separator2,
			XmNleftAttachment,	(XtArgVal) XmATTACH_WIDGET,
                        XmNleftWidget,          (XtArgVal) link_count_label,
			NULL);
		XtVaSetValues (separator3,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,	        (XtArgVal) size,
			XmNleftAttachment,	(XtArgVal) XmATTACH_FORM,
			XmNrightAttachment,	(XtArgVal) XmATTACH_FORM,
			NULL);
		XtVaSetValues (owner_label,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) separator3,
			XmNleftAttachment,	(XtArgVal) XmATTACH_FORM,
			NULL);
		XtVaSetValues (owner,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) separator3,
			XmNleftAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNleftWidget,		(XtArgVal) owner_label,
			XmNrightAttachment, 	(XtArgVal) XmATTACH_WIDGET,
			XmNrightWidget, 	(XtArgVal) group_str_label,
			NULL);
		XtVaSetValues (group_str_label,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) separator3,
			XmNleftAttachment,	(XtArgVal) XmATTACH_POSITION,
			XmNleftPosition,	(XtArgVal) 50,
/*
			XmNleftAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNleftWidget,		(XtArgVal) owner,
*/
			NULL);
		XtVaSetValues (group_str,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) separator3,
			XmNleftAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNleftWidget,		(XtArgVal) group_str_label,
			NULL);
		XtVaSetValues (separator4,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,	        (XtArgVal) owner,
			XmNleftAttachment,	(XtArgVal) XmATTACH_FORM,
			XmNrightAttachment,	(XtArgVal) XmATTACH_FORM,
			NULL);
		XtVaSetValues (last_modified_label,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) separator4,
			XmNleftAttachment,	(XtArgVal) XmATTACH_FORM,
			NULL);
		XtVaSetValues (last_modified,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) separator4,
			XmNleftAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNleftWidget,		(XtArgVal) last_modified_label,
			NULL);
		XtVaSetValues (separator5,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,	        (XtArgVal) last_modified_label,
			XmNleftAttachment,	(XtArgVal) XmATTACH_FORM,
			XmNrightAttachment,	(XtArgVal) XmATTACH_FORM,
			NULL);
		XtVaSetValues (perms_label,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) separator5,
			XmNleftAttachment,	(XtArgVal) XmATTACH_FORM,
			NULL);
		XtVaSetValues (perms_rowcol,
			XmNtopAttachment,	(XtArgVal) XmATTACH_WIDGET,
			XmNtopWidget,		(XtArgVal) perms_label,
			XmNleftAttachment,	(XtArgVal) XmATTACH_FORM,
			XmNrightAttachment,	(XtArgVal) XmATTACH_FORM,
                        XmNbottomAttachment,    (XtArgVal) XmATTACH_FORM,
			NULL);

               /* save all these component widgets in info_dialog_widgets */
               dpi -> idw = (info_dialog_widgets *) XtCalloc (1, sizeof (info_dialog_widgets));
               dpi -> idw -> filename = filename;
               dpi -> idw -> size = size;
               dpi -> idw -> owner = owner;
               dpi -> idw -> group_str = group_str;
               dpi -> idw -> description = description;
	       dpi -> idw -> link_count = link_count;
	       dpi -> idw -> last_modified = last_modified;
               dpi -> idw -> empty_label = empty_label;
               dpi -> idw -> ur_toggle = ur_toggle;
               dpi -> idw -> uw_toggle = uw_toggle;
               dpi -> idw -> ux_toggle = ux_toggle;
               dpi -> idw -> gr_toggle = gr_toggle;
               dpi -> idw -> gw_toggle = gw_toggle;
               dpi -> idw -> gx_toggle = gx_toggle;
               dpi -> idw -> or_toggle = or_toggle;
               dpi -> idw -> ow_toggle = ow_toggle;
               dpi -> idw -> ox_toggle = ox_toggle;
	}
        else
        {   /* get all these widgets back out of dpi */
               filename = dpi -> idw -> filename;
               size = dpi -> idw -> size;
               owner = dpi -> idw -> owner;
               group_str = dpi -> idw -> group_str;
               description = dpi -> idw -> description;
               empty_label = dpi -> idw -> empty_label;
	       link_count = dpi -> idw -> link_count;
	       last_modified = dpi -> idw -> last_modified;
               ur_toggle = dpi -> idw -> ur_toggle;
               uw_toggle = dpi -> idw -> uw_toggle;
               ux_toggle = dpi -> idw -> ux_toggle;
               gr_toggle = dpi -> idw -> gr_toggle;
               gw_toggle = dpi -> idw -> gw_toggle;
               gx_toggle = dpi -> idw -> gx_toggle;
               or_toggle = dpi -> idw -> or_toggle;
               ow_toggle = dpi -> idw -> ow_toggle;
               ox_toggle = dpi -> idw -> ox_toggle;
        }

	/* whether just created or new, need to set all the following */
	mode_changed = False;

	XtVaSetValues (dialog, XmNuserData, dpi, NULL);

	/* set file name */
#ifdef  FILENAME_IS_EDITABLE
	XmTextSetString (filename, dpi -> file_selected);
#else
	xm_file_selected = XmStringCreateSimple (dpi -> file_selected);
	XtVaSetValues (filename, XmNlabelString, xm_file_selected, NULL);
	XmStringFree (xm_file_selected);
#endif /* FILENAME_IS_EDITABLE */

	/* find the file in the DirMgr */
	DirectoryMgrGotoNamedItem (dpi -> directory_manager, 
				    dpi -> file_selected);
	de = DirectoryMgrCurrentEntry (dpi -> directory_manager);
	
	/* set size */
	long_size = FIFileSize (DirEntryActualInfo (de));
	sprintf (str_size, "%ld", long_size);
	xm_size = XmStringCreateSimple (str_size);
	XtVaSetValues (size, XmNlabelString, xm_size, NULL);
	XmStringFree (xm_size);

	/* set owner */
	uid = FIUserID (DirEntryActualInfo (de));
	pw_entry = getpwuid (uid);
	xm_pw_name = XmStringCreateSimple (pw_entry -> pw_name);
	XtVaSetValues (owner, XmNlabelString, xm_pw_name, NULL);
	XmStringFree (xm_pw_name);

	/* set group */
	uid = FIGroupID (DirEntryActualInfo (de));
	grp_entry = getgrgid (uid);
	xm_grp_name = XmStringCreateSimple (grp_entry -> gr_name);
	XtVaSetValues (group_str, XmNlabelString, xm_grp_name, NULL);
	XmStringFree (xm_grp_name);

	/* set description */
        xm_description = XmStringCreateSimple (dpi -> description);
	XtVaSetValues (description, XmNlabelString, xm_description, NULL);
        XmStringFree (xm_description);

	/* set last modified time */
	mod_time = FILastModify (DirEntryActualInfo (de));
	/* use Locale time format */
	strftime (mod_time_buf, sizeof (mod_time_buf), "%c", localtime (&mod_time));
	xm_mod_time = XmStringCreateSimple (mod_time_buf);
	XtVaSetValues (last_modified, XmNlabelString, xm_mod_time, NULL);
	XmStringFree (xm_mod_time);

	/* set link count */
	link_number = FILinkNumber (DirEntryActualInfo (de));
	sprintf (link_number_buf, "%hd", link_number);
	xm_link_number = XmStringCreateSimple (link_number_buf);
	XtVaSetValues (link_count, XmNlabelString, xm_link_number, NULL);
	XmStringFree (xm_link_number);

	/* set permissions */
	permissions = FIProt (DirEntryActualInfo (de));
        XtVaSetValues (empty_label, XmNlabelString, xm_empty, NULL);

	state = (permissions & S_IRUSR) ? True : False;
        XtVaSetValues (ur_toggle,
                        XmNlabelString, xm_empty,
			XmNset, 	(XtArgVal) state,
                        NULL);

	state = (permissions & S_IWUSR) ? True : False;
        XtVaSetValues (uw_toggle,
                        XmNlabelString, xm_empty,
			XmNset, 	(XtArgVal) state,
                        NULL);

	state = (permissions & S_IXUSR) ? True : False;
        XtVaSetValues (ux_toggle,
                        XmNlabelString, xm_empty,
			XmNset, 	(XtArgVal) state,
                        NULL);

	state = (permissions & S_IRGRP) ? True : False;
        XtVaSetValues (gr_toggle,
                        XmNlabelString, xm_empty,
			XmNset, 	(XtArgVal) state,
                        NULL);

	state = (permissions & S_IWGRP) ? True : False;
        XtVaSetValues (gw_toggle,
                        XmNlabelString, xm_empty,
			XmNset, 	(XtArgVal) state,
                        NULL);

	state = (permissions & S_IXGRP) ? True : False;
        XtVaSetValues (gx_toggle,
                        XmNlabelString, xm_empty,
			XmNset, 	(XtArgVal) state,
                        NULL);

	state = (permissions & S_IROTH) ? True : False;
        XtVaSetValues (or_toggle,
                        XmNlabelString, xm_empty,
			XmNset, 	(XtArgVal) state,
                        NULL);

	state = (permissions & S_IWOTH) ? True : False;
        XtVaSetValues (ow_toggle,
                        XmNlabelString, xm_empty,
			XmNset, 	(XtArgVal) state,
                        NULL);

	state = (permissions & S_IXOTH) ? True : False;
        XtVaSetValues (ox_toggle,
                        XmNlabelString, xm_empty,
			XmNset, 	(XtArgVal) state,
                        NULL);

        XtManageChild (dialog);
}

