/*************************************************************************
 * File: $Source: /usr/usrs/xsource/xmfm/RCS/runcb.c,v $
 * Author: Jan Newmarch
 * Last modified: $Date: 1993/04/01 04:15:46 $
 * Version: $Revision: 1.11 $
 * Purpose: This file handles the run operations from the system menu
 *
 * Revision history:
 *      22 Aug 92      Separate run dialogs per pane (needed in case a pane
 *                     gets iconified or destroyed
 *	16 Oct 92	caddr_t changed to XtPointer
 *       3 Nov 92      lint-ed
 *	 1 Dec 92      toggle buttons run_in_xterm etc got confused with
 *		       multiple panes (global, not pane specific)
 *	 5 Dec 92      added $0 expansion to run command
 *	25 Mar 93	all pointer vbls grounded on definition, many
 *			XtMallocs changed to XtCalloc to trap pointer bugs
 ************************************************************************/ 

#include "copyright.h"

/*************************************************************************
 * System includes
 ************************************************************************/ 
#include <stdlib.h>
#include <stdio.h> 

#include <Xm/CascadeB.h>     
#include <Xm/FileSB.h>      
#include <Xm/Label.h>
#include <Xm/PushB.h>    
#include <Xm/RowColumn.h>    
#include <Xm/SelectioB.h>    

/*************************************************************************
 * Local includes
 ************************************************************************/ 
#include "Directory.h"
#include "DirMgr.h"
#include "filecb.h"
#include "types.h"
#include "xmvararg.h"
#include "debug_malloc.h"

/*************************************************************************
 * Functions exported
 ************************************************************************/ 
extern void RunCommandCB (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer client_data, XtPointer call_data
#endif
);
extern void RunXtermCB (
#ifdef UseFunctionPrototypes
	Widget w, XtPointer client_data, XtPointer call_data
#endif
);

/*************************************************************************
 * Variables exported
 ************************************************************************/ 

/*************************************************************************
 * Extern variables
 ************************************************************************/ 

/*************************************************************************
 * Extern functions
 ************************************************************************/ 
extern void GotoCurrentDir (
#ifdef UseFunctionPrototypes
	Widget w
#endif
);
extern void do_command (
#ifdef UseFunctionPrototypes
	char *command, int run_in_xterm, int pause_after_exec, dir_pane_info *dpi
#endif
);
extern void expand_command (
#ifdef UseFunctionPrototypes
	char **command, char **argv, int argc, char *action
#endif
);

/*************************************************************************
 * Forward functions
 ************************************************************************/ 

/*************************************************************************
 * Local variables
 ************************************************************************/ 
static Boolean run_in_xterm = False;
static Boolean pause_after_exec = False;

#define LIST_SIZE 10

#ifndef min
#define min(x,y)	((x) < (y) ? (x) : (y))
#endif

/*************************************************************************
 * Function: ButtonToggledCB ()
 * Purpose: a check button has been changed
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition; 
 * Postcondition: appropriate Boolean for Check box is updated
 ************************************************************************/

/* ARGSUSED */ 
static void
ButtonToggledCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)
	Widget w;
	XtPointer client_data;
	XtPointer call_data;

#endif
{
	/* toggle value of correct Boolean */
	if ( (int) client_data == 0)
		run_in_xterm = (run_in_xterm ? False : True);
	else 	pause_after_exec = (pause_after_exec ? False : True);
}

/*************************************************************************
 * Function: OkCB ()
 * Purpose: run the command entered
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition: Ok button pressed in Command dialog
 * Postcondition: user command executed, dialog killed
 ************************************************************************/ 

/* ARGSUSED */
static void
OkCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)
	Widget w;
	XtPointer client_data;
	XtPointer call_data;

#endif
{
	XmString xmstr_command = NULL;
	String command = NULL;
	String new_command = NULL;
	int i, new_size, curr_size;
	XmString new_list[LIST_SIZE], *curr_list = NULL;
        dir_pane_info *dpi = NULL;
	Widget run_in_xterm_btn = NULL, pause_after_exec_btn = NULL;

	XtVaGetValues (w,
			XmNtextString, &xmstr_command,
                        XmNuserData, &dpi,
			NULL);

	XmStringGetLtoR (xmstr_command, XmSTRING_DEFAULT_CHARSET, &command);

	/* find state of toggle buttons */
	run_in_xterm_btn = (dpi -> run_cmd_buttons) [0];
	pause_after_exec_btn = (dpi -> run_cmd_buttons) [1];
	XtVaGetValues (run_in_xterm_btn, XmNset, &run_in_xterm, NULL);
	XtVaGetValues (pause_after_exec_btn, XmNset, &pause_after_exec, NULL);

	GotoCurrentDir (w);

	/* do any parameter expansion for $0 */
	if (dpi -> file_selected != NULL)
		expand_command (&new_command, (&dpi -> file_selected),
				1, command);
	else	expand_command (&new_command, NULL, 0, command);

	/* and run it */
	do_command (new_command, run_in_xterm, pause_after_exec, dpi);
	
	XtFree (command);
	XtFree (new_command);

	XtUnmanageChild (w);

	/* before we leave this: add the command into the list of
	  previous commands, as long as it isn't already there */
	XtVaGetValues (w,
			XmNlistItems, &curr_list,
			XmNlistItemCount, &curr_size,
			NULL);
	for (i = 0; i < curr_size; i++)
		if (XmStringCompare (xmstr_command, curr_list[i]))
			/* already in list */
			return;
	/* new elmt. put in front, and shuffle up the rest */
	new_list[0] = xmstr_command;
	new_size = min (curr_size + 1, LIST_SIZE);
	for (i = 1; i < new_size; i++)
		new_list[i] = XmStringCopy (curr_list[i - 1]);
	XtVaSetValues (w,
			XmNlistItems, new_list,
			XmNlistItemCount, (XtArgVal) new_size,
			NULL);
	/* and garbage collect */
/*
	for (i = 0; i < new_size; i++)
		XmStringFree (new_list[i]);
*/
}

/*************************************************************************
 * Function: RunCommandCB ()
 * Purpose: run a user command in current dir
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition; Command button pressed in Run menu
 * Postcondition: user command dialog showing
 ************************************************************************/ 

/* ARGSUSED */
void
RunCommandCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
	Widget		w;		/*  widget id		*/ 
	XtPointer		client_data;	/*  data from applicaiton   */ 
	XtPointer		call_data;	/*  data from widget class  */ 

#endif
{	 
	dir_pane_info *dpi = NULL;
	Widget	run_command = NULL;
	Widget	rowcol = NULL;
	XmString xmstr_buttons[2];
	XmString xmstr_prompt = NULL;
	Widget help_button = NULL, apply_button = NULL;
	static XmString null_str = NULL;
	WidgetList buttons = NULL;

        XtVaGetValues (w, XmNuserData, &dpi, NULL);
        run_command = dpi -> run_dialog;
	/* create new dialog if not already existing */
	if (run_command == NULL)
	{
		xmstr_prompt = XmStringCreateSimple ("Enter run command:");
		run_command = XmVaCreateSelectionDialog (w,
					"run command",
					XmNpromptString, xmstr_prompt,
	        			XmNuserData, dpi,
					NULL);

                /* save this back into dpi */
                dpi -> run_dialog = run_command;

		XmStringFree (xmstr_prompt);

		XtAddCallback (run_command, XmNokCallback, OkCB, NULL);
		/* no cancel callback - default pops it down */

		/* remove the help and cancel */
		help_button = XmSelectionBoxGetChild (run_command, 
				XmDIALOG_HELP_BUTTON);
		XtUnmanageChild (help_button);
		apply_button = XmSelectionBoxGetChild (run_command, 
				XmDIALOG_APPLY_BUTTON);
		XtUnmanageChild (apply_button);
	
		xmstr_buttons[0] = XmStringCreateSimple ("Run in xterm");
		xmstr_buttons[1] = XmStringCreateSimple ("Pause after exec");
	
		null_str = XmStringCreateSimple ("");

		rowcol = XmVaCreateSimpleCheckBox (run_command, "rowcol",
				ButtonToggledCB,
				XmNbuttonCount, (XtArgVal) 2,
				XmNbuttons, xmstr_buttons,
				XmVaCHECKBUTTON, xmstr_buttons[0],
					NULL, NULL, NULL,
				XmVaCHECKBUTTON, xmstr_buttons[1],
					NULL, NULL, NULL,
				NULL);
		XtManageChild (rowcol);

		/* save the run_in_xterm/pause_after_exec buttons in dpi */
		XtVaGetValues (rowcol, XmNchildren, &buttons, NULL);
		dpi -> run_cmd_buttons = buttons;

		XmStringFree (xmstr_buttons[0]);
		XmStringFree (xmstr_buttons[1]);
	}

	/* ensure text field is empty for new command */
/*	both these methods dump core
	XtVaSetValues (run_command,
			XmNtextString, null_str,
			NULL);
	XmCommandSetValue (run_command, null_str);
*/

      	XtManageChild (run_command);
}
 
/*************************************************************************
 * Function: RunXtermCB()
 * Purpose: run a new xterm
 * In parameters: w, client_data, call_data
 * Out parameters:
 * Precondition; Xterm selected from Run menu
 * Postcondition: new xterm running
 ************************************************************************/ 

/* ARGSUSED */
void
RunXtermCB 
#ifdef UseFunctionPrototypes
	(Widget w, XtPointer client_data, XtPointer call_data)
#else
	(w, client_data, call_data)  
	Widget		w;		/*  widget id		*/ 
	XtPointer		client_data;	/*  data from applicaiton   */ 
	XtPointer		call_data;

#endif
{
        dir_pane_info *dpi = NULL;
	char command[1024];

        XtVaGetValues (w, XmNuserData, &dpi, NULL);

	GotoCurrentDir (w);

#ifdef DEBUG
	fprintf (stderr, "xterm, display is %s\n",
		DisplayString (XtDisplay (w)));
#endif
	strcpy (command, "xterm -display ");
	strcat (command, DisplayString (XtDisplay (w)));
	do_command (command, False, False, dpi);
}

