/*************************************************************************
 * File: $Source: /usr/usrs/xsource/xmfm/RCS/types.h,v $
 * Author: Jan Newmarch
 * Last modified: $Date: 1993/02/01 03:28:24 $
 * Version: $Revision: 1.9 $
 * Purpose:
 *
 * Revision history:
 *	11 Aug 92	added description field to file_action
 *      22 Ayg 92       added run_dialog and info_dialog as per pane stuff
 *	17 Dec 92	added being_destroyed, ref_count to dir_pane_info
 ************************************************************************/ 

#include "copyright.h"

#ifndef _XMFM_TYPES_H
#define _XMFM_TYPES_H

#include <Xm/Xm.h>

#include "DirMgr.h"

#define FSM_LENGTH 1024

/*****************************************************************
 * Data type:
 * Used by:
 * Purpose:
 * Description:
 *****************************************************************/

/*****************************************************************
 * Data type: action_pair
 * Used by: do_command(), FileToolbarButtonPressed()
 * Purpose: define a possible action that can be taken on a file
 * Description: It contains a label to be displayed in an action button,
 *		the action itself,
 *		Booleans to control the exection environment of the command,
 *		and a pointer to another action_pair record
 *****************************************************************/
typedef struct action_pair
{
	char			*action_label;
	char			*action;
	char			*prompt;
	char			*confirm;
	Bool			run_in_xterm;
	Bool			pause_after_exec;
	Bool			has_prompt;
	Bool			has_confirm;
	struct action_pair	*next;
}	action_pair;


/*****************************************************************
 * Data type: file_action
 * Used by:
 * Purpose: this is used to store the actions associated with each 
 *	file/directory/executable.
 * Description: This is keyed on the file name pattern (eg *.c) and
 *	the file type (eg 'f'). It keeps as data a pixmap and a
 *	description of the type (eg 'C source code'). It also has
 *	a linked list of action records.
 *****************************************************************/
typedef struct file_action
{
	char			file_type;
	char			*name_pattern;
	char			fsm_ptr[FSM_LENGTH];
	char			*pixmap;
	char			*description;
	action_pair		*actions;
	struct file_action	*next;
}	file_action;

#ifdef OLD_CODE
/*****************************************************************
 * Data type: aWidgetList
 * Used by:
 * Purpose: Form a list of widgets
 * Description: a linked list of widgets.
 *****************************************************************/
typedef struct aWidgetList
{
	Widget			widget;
	struct aWidgetList	*next;
}	aWidgetList;
#endif /* OLD_CODE */

/*****************************************************************
 * Data type: info_dialog_widgets
 * Used by:
 * Purpose: An info dialog is sufficiently complex that we dont want
 *	to spend lots of time creating it every time we need one. So we
 *	keep them around (wasting space instead). Each time we reuse
 *	one, all of its following fields need to be reset
 * Description:
 *****************************************************************/
typedef struct info_dialog_widgets
{  Widget	
		filename,
		size,
		owner,
		group_str,
		description,
		link_count,
		last_modified,
		empty_label,
		ur_toggle,
		uw_toggle,
		ux_toggle,
		gr_toggle,
		gw_toggle,
		gx_toggle,
		or_toggle,
		ow_toggle,
		ox_toggle;
}      info_dialog_widgets;


/*****************************************************************
 * Data type: dir_pane-info
 * Used by: almost everything.
 * Purpose: it carries around "global" data to all callbacks that need
 *	it.  It is placed in the user data of most widgets (except file
 *	buttons, which use pane_button_info, and toolbar buttons which
 *	use tool_button_info). Most callbacks get this structure from
 *	their userData first off. There is one of these per directory
 *	showing.
 * Description:
 *****************************************************************/
typedef struct dir_pane_info
{	
	/* label at top showing current dir */
	Widget		directory_label;

	/* two sets of toolbar buttons */
	Widget		*files_toolbar;
	Widget		*dirs_toolbar;

	/* three lists of file/dir/exec buttons in the panes */
	WidgetList	files_buttons;
	WidgetList	executables_buttons;
	WidgetList	dirs_buttons;

	int		files_buttons_last;
	int		dirs_buttons_last;
	int		executables_buttons_last;

	int		files_buttons_max;
	int		dirs_buttons_max;
	int		executables_buttons_max;
	

	/* and the panes they are in */
	Widget		files_pane;
	Widget		dirs_pane;
	Widget		executables_pane;

	/* list of all files in current dir matching selection pattern */
	DirectoryMgr	*directory_manager;

	/* currently selected file/dir/exec button */
	char		*file_selected;		/* name */
	Widget		file_button_selected;	/* widget button */
	char		*description;		/* what sort of file */

	char		*executable_selected;	/* unused */
	char		*dir_selected;		/* unused */

	/* top of this widget tree */
	Widget		toplevel;

	/* filters used in file matching */
	char		*file_filter;
	char		file_filter_regexp[FSM_LENGTH];
	char		*dir_filter;
	char		dir_filter_regexp[FSM_LENGTH];
	char		*executable_filter;
	char		executable_filter_regexp[FSM_LENGTH];
	char		all_filter;

	/* widgets showing the filter info */
	Widget		file_filter_label;
	Widget		dir_filter_label;
	Widget		executable_filter_label;

        /* dialogs that are non-transient, must be kept on a per pane basis */
        Widget          run_dialog;
        Widget          info_dialog;
        WidgetList      run_cmd_buttons;
        info_dialog_widgets *idw;

	/* if a child is forked, it may have a pointer to this even after
	   the pane this belongs to is destroyed. To avoid illegal
	   references through this, keep track of aliveness (see refresh.c)
	*/
	Bool		being_destroyed;
	int		reference_count;
}	dir_pane_info;

/*****************************************************************
 * Data type: pane_button_info
 * Used by:
 * Purpose: each button in a pane (eg an executable button) needs
 *	both the general dpi info but also display info
 * Description:
 *****************************************************************/
typedef struct pane_button_info
{	char		*name;
	char		file_type;
	dir_pane_info	*dpi;
	Pixmap		pixmap;
	Pixmap		pixmap_reversed;
	GC		gc;
	GC		gc_reversed;
}	pane_button_info;

/*****************************************************************
 * Data type: tool_button_info
 * Used by:
 * Purpose: each button in a toolbar needs dpi info. 
 *	Also, it requires runtime info for when it is pressed
 * Description:
 *****************************************************************/
typedef struct tool_button_info
{	char		*action;
	dir_pane_info	*dpi;
	Bool		run_in_xterm;
	Bool		pause_after_exec;
        Bool            has_prompt;
        char            *prompt;
	Bool		has_confirm;
	char		*confirm;
}	tool_button_info;

/* compatability with Apollo? */
#ifndef S_ISDIR
#define S_ISDIR(m)	((m) & S_IFDIR)
#endif

#endif /* _XMFM_TYPES_H */
