/*************************************************************************
 * File: $Source: /usr/usrs/xsource/xmfm/RCS/utils.c,v $
 * Author: Jan Newmarch
 * Last modified: $Date: 1993/04/01 04:15:48 $
 * Version: $Revision: 1.10 $
 * Purpose: general purpose routines of use in various places
 *
 * Revision history:
 *	6 Aug 92	BusyCursor covers whole of a pane, rather than 1 widget
 *      22 Aug 92       NULL terminated string for tilde expansion
 *       3 Nov 92       lint-ed
 *	30 Nov 92	added GetCWD
 *	 3 Dec 92	added Widget argument to XtChdir for error dialogs
 *	 5 Dec 92	looks up SHELL in environment
 *	18 Dec 92	cleaned compile under g++
 *	25 Mar 93	all pointer vbls grounded on definition, many
 *			XtMallocs changed to XtCalloc to trap pointer bugs
 ************************************************************************/ 

#include "copyright.h"

/*************************************************************************
 * System includes
 ************************************************************************/ 
#include <sys/param.h>
#include <string.h>
#include <Xm/Xm.h>
#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#include <pwd.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/wait.h>

/*************************************************************************
 * Local includes
 ************************************************************************/ 
#include "DirMgr.h"
#include "types.h"
#include "const.h"
#include "debug_malloc.h"

/*************************************************************************
 * Functions exported
 ************************************************************************/ 
extern Bool XtChdir (
#ifdef UseFunctionPrototypes
	char *dir, Widget w
#endif
);
extern void BusyCursor (
#ifdef UseFunctionPrototypes
	Widget toplevel
#endif
);
extern void UnBusyCursor (
#ifdef UseFunctionPrototypes
	Widget toplevel
#endif
);
extern void GotoCurrentDir (
#ifdef UseFunctionPrototypes
	Widget w
#endif
);

extern char *GetCWD (
#ifdef UseFunctionPrototypes
	char *, int
#endif
);

/*************************************************************************
 * Variables exported
 ************************************************************************/ 

/*************************************************************************
 * Extern variables
 ************************************************************************/ 

/*************************************************************************
 * Extern functions
 ************************************************************************/ 
extern char *getlogin ();   /* gcc (?) has no prototype for this */
extern void ErrorDialog (
#ifdef UseFunctionPrototypes
	Widget w, char *str
#endif
);

extern void child_died (
#ifdef UseFunctionPrototypes
	int
#endif
);

/*************************************************************************
 * Forward functions
 ************************************************************************/ 
char *getcwd (
#ifdef UseFunctionPrototypes
         char *, size_t
#endif
);
/*************************************************************************
 * Local variables
 ************************************************************************/ 
static Window busy_window = 0; /* is this an "illegal" value? */
#define IS_WHITE_SPACE(ch)	((ch) == ' ' || (ch) == '\t' || (ch) == '\n')

static Cursor watch = 0; /* is this an "illegal" value to test against? */ 
static unsigned long width = 0;
static unsigned long height = 0;

/*
void (*signal())()
{
   fprintf (stderr, "signalling\n");
}
*/

/*************************************************************************
 * Function        : BusyCursor ()
 * Purpose         : make the cursor turn busy for this window
 * In parameters   : w
 * Out parameters  :
 * Side effects    :
 * Function returns:
 * Precondition    :
 * Postcondition   : cursor over current window is busy
 ************************************************************************/ 
void
BusyCursor 
#ifdef UseFunctionPrototypes
	(Widget toplevel)
#else
	(toplevel)
	Widget toplevel;

#endif
{  
        unsigned long valuemask;
        XSetWindowAttributes attributes;
 
       	if(!watch)
               	watch = XCreateFontCursor(XtDisplay(toplevel),XC_watch);
	if (!width)
		width = WidthOfScreen (XtScreen (toplevel));
	if (!height)
		height = HeightOfScreen (XtScreen (toplevel));

        /* Ignore device events while the busy cursor is displayed. */
        valuemask = CWDontPropagate | CWCursor;
        attributes.do_not_propagate_mask =  (KeyPressMask | KeyReleaseMask |
           ButtonPressMask | ButtonReleaseMask | PointerMotionMask);
        attributes.cursor = watch;

        /* The window will be as big as the display screen, and clipped by
           its own parent window, so we never have to worry about resizing */
        /* on multiple clicks, this function can get called multiple times.
           check that we haven't already got such a window mapped */
        if (busy_window != 0)
                return;
        busy_window = XCreateWindow(XtDisplay(toplevel), XtWindow(toplevel), 0, 0,
                       65535, 65535, (unsigned int) 0, CopyFromParent, InputOnly,
                       CopyFromParent, valuemask, &attributes);
        XMapWindow (XtDisplay (toplevel), busy_window);
   

        XmUpdateDisplay(toplevel);

}

/*************************************************************************
 * Function        : UnBusyCursor ()
 * Purpose         : set it back again
 * In parameters   : w
 * Out parameters  :
 * Side effects    :
 * Function returns:
 * Precondition    :
 * Postcondition   : cursor is back to previous cursor
 ************************************************************************/ 
void
UnBusyCursor 
#ifdef UseFunctionPrototypes
	(Widget toplevel)
#else
	(toplevel)
	Widget toplevel;

#endif
{
        /* check that we have a window to destroy */
        if (busy_window == 0)
               return;       
        XDestroyWindow (XtDisplay (toplevel), busy_window);
        busy_window = 0;
}

/*************************************************************************
 * Function        : XtChdir ()
 * Purpose         : perform a chdir and report on any error. also does
 *	tilde expansion
 * In parameters   : dir
 * Out parameters  :
 * Side effects    :
 * Function returns: success or fail
 * Precondition    :
 * Postcondition   : application now in new dir (success)
 *		error dialog posted (failure)
 ************************************************************************/ 
Bool
XtChdir 
#ifdef UseFunctionPrototypes
	(char *dir, Widget w)
#else
	(dir, w)
	char *dir;
	Widget w;

#endif
{	char warning[MAXPATHLEN + 22];
	char full_dir[MAXPATHLEN + 2];
	char name_buf[MAXPATHLEN + 2];
	char *name = NULL;
	char *orig_dir = dir;
        struct passwd *pw_ent = NULL;

	while (IS_WHITE_SPACE (*dir))
		dir++;
	/* should it undergo ~ expansion?
	*/ 
	if (*dir == '~')
	{	/* Found a tilde. This isn't full globbing.
		   We only have these cases:
			~
			~/...
			~name
			~name/...
		*/
		/* move past ~ */
		dir++;
		if ( *dir == '\0' || *dir == '/')
		{	/* name is login name */
			name = getlogin ();
			/* dir now points to next char after user name */
		}
		else	/* name is someone else */
		{	char *p = dir;

			while ( ! IS_WHITE_SPACE (*p) &&
				*p != '/' && *p != '\0')
				p++;
                        /* leave space for \0 */
			strncpy (name_buf, dir, (int) (p - dir));
                        name_buf[(int) (p - dir)] = '\0';
			name = name_buf;

			dir = p;
			/* dir now points to next char after user name */
		}
		/* put in users home dir */
		pw_ent = getpwnam (name);
		if (pw_ent == NULL)
			/* no such user, restore original pattern */
			dir = orig_dir;
		else
		{
			strcpy (full_dir, pw_ent -> pw_dir);
			/* and what followed after */
			strcat (full_dir, dir);
			/* set dir to point to all of this */
			dir = full_dir;
		}
	}
	if (chdir (dir) == -1)
	{
		strcpy (warning, "cannot chdir to ");
		strcat (warning, dir);
		ErrorDialog (w, warning);
		return False;
	}
	return True;
}

/*************************************************************************
 * Function        : GotoCurrentDir ()
 * Purpose         : make sure we are in current dir
 * In parameters   : w
 * Out parameters  :
 * Side effects    :
 * Function returns:
 * Precondition    :
 * Postcondition   : application now in new dir
 ************************************************************************/ 
void
GotoCurrentDir 
#ifdef UseFunctionPrototypes
	(Widget w)
#else
	(w)
	Widget w;

#endif
{	dir_pane_info *dpi = NULL;

	XtVaGetValues (w,
			XmNuserData, &dpi,
			NULL);
	XtChdir (DirectoryPath (DirectoryMgrDir (dpi -> directory_manager)),
			dpi -> toplevel);
}

/*************************************************************************
 * Function        : GetCWD
 * Purpose         : get current dir
 * In parameters   : path, size
 * Out parameters  :
 * Side effects    :
 * Function returns:
 * Precondition    :
 * Postcondition   : application now in new dir
 ************************************************************************/ 
extern char *GetCWD
#ifdef UseFunctionPrototypes
	(char *path, int size)
#else
	(path, size)
	char *path;
	int size;
#endif
{	char *p = NULL;

	/* On my Sun this uses popen() and wait(). I already have a handler
	   on SIGCHLD, and they conflict somehow. I have no idea
	   why, but I have to turn off my sighandler first
	*/
	signal (SIGCHLD, SIG_IGN);
#ifdef NO_GETCWD
	p = getwd (path);
#else
	p = getcwd (path, size);
#endif
	signal (SIGCHLD, child_died);
	return p;
}

/*************************************************************************
 * Function        : XSystem
 * Purpose         : make a system call, but close all open files for X
 * In parameters   : call, w
 * Out parameters  :
 * Side effects    :
 * Function returns: 
 * Precondition    :
 * Postcondition   : 
 ************************************************************************/ 

/* ARGSUSED */
/* w isnt used, but may be later if I trap output from the command */
void XSystem
#ifdef UseFunctionPrototypes
       (char * command, Widget w)
#else
       (command, w)
       char *command;
       Widget w;
#endif
{
       int n;
       pid_t pid;
       int status;
       static char *shell = NULL;
       static char *shell_basename = NULL;

	/* turn off current signal */
	signal (SIGCHLD, SIG_IGN);

	/* find out what the user's shell is */
	if (shell == NULL)
		if ((shell = getenv ("SHELL")) == NULL)
			shell = "/bin/sh";
	if (shell_basename == NULL)
	{	shell_basename = shell + strlen (shell);
		while(	shell_basename > shell && *shell_basename != '/')
			shell_basename--;
	}

#ifdef DEBUG
       fprintf (stderr, "executing command %s\n", command);
#endif

       if ((pid = fork ()) == -1)
              return;
       if (pid == 0)
       {       /* child */
              for (n = 0; n < sysconf (_SC_OPEN_MAX); n++)
			close (n);
/*
                     fcntl (n, F_SETFD, 1);
*/
              execl (shell, shell_basename, "-c", command, (char *) 0);
              fprintf (stderr, "execl %s failed\n", command);
       }
       wait (&status);
       signal (SIGCHLD, child_died);
}
